@@ -42,8 +42,10 @@
 
 #  define flockfile(f)
 #  define funlockfile(f)
+#  undef putc_unlocked
 #  define putc_unlocked(c, f) putc((c), (f))
 #  define fputs_unlocked(s, f) fputs((s), (f))
+#  undef fwrite_unlocked
 #  define fwrite_unlocked(b, s, n, f) fwrite((b), (s), (n), (f))
 # endif /* No thread safe i/o */
 #endif /* !_LIBC */
@@ -99,6 +101,17 @@ typedef FILE *argp_fmtstream_t;
 #ifndef __const
 #define __const const
 #endif
+
+/* FIXME: We could use a configure test to check for __attribute__,
+ * just like lsh does. */
+#ifndef PRINTF_STYLE
+# if __GNUC__ >= 2
+#  define PRINTF_STYLE(f, a) __attribute__ ((__format__ (__printf__, f, a)))
+# else
+#  define PRINTF_STYLE(f, a)
+# endif
+#endif
+
 
 struct argp_fmtstream
 {
@@ -140,10 +153,10 @@ extern void argp_fmtstream_free (argp_fmtstream_t __fs);
 
 extern ssize_t __argp_fmtstream_printf (argp_fmtstream_t __fs,
 				       __const char *__fmt, ...)
-     __attribute__ ((__format__ (printf, 2, 3)));
+     PRINTF_STYLE(2,3);
 extern ssize_t argp_fmtstream_printf (argp_fmtstream_t __fs,
 				      __const char *__fmt, ...)
-     __attribute__ ((__format__ (printf, 2, 3)));
+     PRINTF_STYLE(2,3);
 
 extern int __argp_fmtstream_putc (argp_fmtstream_t __fs, int __ch);
 extern int argp_fmtstream_putc (argp_fmtstream_t __fs, int __ch);
