@@ -72,14 +72,6 @@ char *alloca ();
 #include "argp-namefrob.h"
 
 
-#ifndef UNUSED
-# if __GNUC__ >= 2
-#  define UNUSED __attribute__ ((__unused__))
-# else
-#  define UNUSED
-# endif
-#endif
-
 #ifndef _LIBC
 # ifndef __strchrnul
 #  define __strchrnul strchrnul
@@ -194,18 +186,17 @@ static const struct uparam_name uparam_names[] =
   { "header-col",     0, offsetof (struct uparams, header_col) },
   { "usage-indent",   0, offsetof (struct uparams, usage_indent) },
   { "rmargin",        0, offsetof (struct uparams, rmargin) },
-  { 0 }
+  { 0, 0, 0 }
 };
 
 /* Read user options from the environment, and fill in UPARAMS appropiately.  */
 static void
 fill_in_uparams (const struct argp_state *state)
 {
-  /* FIXME: Can we define var as a pointer to _unsigned_ char,
-   * or will that clash with the getenv() prototype? */
-  const char *var = getenv ("ARGP_HELP_FMT");
+  /* FIXME: Can we get away without an explicit cast? */
+  const unsigned char *var = (unsigned char *) getenv ("ARGP_HELP_FMT");
 
-#define SKIPWS(p) do { while (isspace ( (unsigned) *p)) p++; } while (0);
+#define SKIPWS(p) do { while (isspace (*p)) p++; } while (0);
 
   if (var)
     /* Parse var. */
@@ -213,14 +204,14 @@ fill_in_uparams (const struct argp_state *state)
       {
 	SKIPWS (var);
 
-	if (isalpha ( (unsigned) *var))
+	if (isalpha (*var))
 	  {
 	    size_t var_len;
 	    const struct uparam_name *un;
 	    int unspec = 0, val = 0;
-	    const char *arg = var;
+	    const unsigned char *arg = var;
 
-	    while (isalnum ( (unsigned) *arg) || *arg == '-' || *arg == '_')
+	    while (isalnum (*arg) || *arg == '-' || *arg == '_')
 	      arg++;
 	    var_len = arg - var;
 
@@ -245,10 +236,10 @@ fill_in_uparams (const struct argp_state *state)
 		else
 		  val = 1;
 	      }
-	    else if (isdigit ( (unsigned) *arg))
+	    else if (isdigit (*arg))
 	      {
 		val = atoi (arg);
-		while (isdigit ( (unsigned) *arg))
+		while (isdigit (*arg))
 		  arg++;
 		SKIPWS (arg);
 	      }
@@ -737,17 +728,19 @@ hol_cluster_is_child (const struct hol_cluster *cl1,
 /* Given the name of a OPTION_DOC option, modifies NAME to start at the tail
    that should be used for comparisons, and returns true iff it should be
    treated as a non-option.  */
+
+/* FIXME: Can we use unsigned char * for the argument? */
 static int
 canon_doc_option (const char **name)
 {
   int non_opt;
   /* Skip initial whitespace.  */
-  while (isspace ( (unsigned) **name))
+  while (isspace ( (unsigned char) **name))
     (*name)++;
   /* Decide whether this looks like an option (leading `-') or not.  */
   non_opt = (**name != '-');
   /* Skip until part of name used for sorting.  */
-  while (**name && !isalnum ( (unsigned) **name))
+  while (**name && !isalnum ( (unsigned char) **name))
     (*name)++;
   return non_opt;
 }
@@ -787,6 +780,7 @@ hol_entry_cmp (const struct hol_entry *entry1,
       int short2 = hol_entry_first_short (entry2);
       int doc1 = odoc (entry1->opt);
       int doc2 = odoc (entry2->opt);
+      /* FIXME: Can we use unsigned char * instead? */
       const char *long1 = hol_entry_first_long (entry1);
       const char *long2 = hol_entry_first_long (entry2);
 
@@ -809,15 +803,18 @@ hol_entry_cmp (const struct hol_entry *entry1,
 	   first, but as they're not displayed, it doesn't matter where
 	   they are.  */
 	{
-	  char first1 = short1 ? short1 : long1 ? *long1 : 0;
-	  char first2 = short2 ? short2 : long2 ? *long2 : 0;
+	  unsigned char first1 = short1 ? short1 : long1 ? *long1 : 0;
+	  unsigned char first2 = short2 ? short2 : long2 ? *long2 : 0;
 #ifdef _tolower
-	  int lower_cmp = _tolower ( (unsigned) first1) - _tolower ( (unsigned) first2);
+	  int lower_cmp = _tolower (first1) - _tolower (first2);
 #else
-	  int lower_cmp = tolower ( (unsigned) first1) - tolower ( (unsigned) first2);
+	  int lower_cmp = tolower (first1) - tolower (first2);
 #endif
 	  /* Compare ignoring case, except when the options are both the
 	     same letter, in which case lower-case always comes first.  */
+	  /* NOTE: The subtraction below does the right thing
+	     even with eight-bit chars: first1 and first2 are
+	     converted to int *before* the subtraction. */
 	  return lower_cmp ? lower_cmp : first2 - first1;
 	}
     }
@@ -1107,7 +1104,17 @@ hol_entry_help (struct hol_entry *entry, const struct argp_state *state,
   int old_wm = __argp_fmtstream_wmargin (stream);
   /* PEST is a state block holding some of our variables that we'd like to
      share with helper functions.  */
+#ifdef __GNUC__
   struct pentry_state pest = { entry, stream, hhstate, 1, state };
+#else /* !__GNUC__ */
+  /* Decent initializers are a GNU extension */
+  struct pentry_state pest;
+  pest.entry = entry;
+  pest.stream = stream;
+  pest.hhstate = hhstate;
+  pest.first = 1;
+  pest.state = state;
+#endif /* !__GNUC__ */
 
   if (! odoc (real))
     for (opt = real, num = entry->num; num > 0; opt++, num--)
@@ -1720,21 +1727,24 @@ char *__argp_basename(char *name)
   return short_name ? short_name + 1 : name;
 }
 
-static const char *
-short_program_name(const struct argp_state *state)
+char *
+__argp_short_program_name(const struct argp_state *state)
 {
   if (state)
     return state->name;
-#if HAVE_PROGRAM_INVOCATION_SHORT_NAME
+#if HAVE_DECL_PROGRAM_INVOCATION_SHORT_NAME
   return program_invocation_short_name;
-#elif HAVE_PROGRAM_INVOCATION_NAME
+#elif HAVE_DECL_PROGRAM_INVOCATION_NAME
   return __argp_basename(program_invocation_name);
-#else
+#else /* !HAVE_DECL_PROGRAM_INVOCATION_NAME */
   /* FIXME: What now? Miles suggests that it is better to use NULL,
      but currently the value is passed on directly to fputs_unlocked,
      so that requires more changes. */
+# if __GNUC__
+#  warning No reasonable value to return
   return "";
-#endif
+# endif /* __GNUC__ */
+#endif /* !HAVE_DECL_PROGRAM_INVOCATION_NAME */
 }
 
 /* Output, if appropriate, a usage message for STATE to STREAM.  FLAGS are
@@ -1748,7 +1758,7 @@ __argp_state_help (const struct argp_state *state, FILE *stream, unsigned flags)
 	flags |= ARGP_HELP_LONG_ONLY;
 
       _help (state ? state->root_argp : 0, state, stream, flags,
-	     short_program_name(state));
+	     __argp_short_program_name(state));
 
       if (!state || ! (state->flags & ARGP_NO_EXIT))
 	{
@@ -1779,7 +1789,7 @@ __argp_error (const struct argp_state *state, const char *fmt, ...)
 
 	  __flockfile (stream);
 
-	  fputs_unlocked (short_program_name(state),
+	  fputs_unlocked (__argp_short_program_name(state),
 			  stream);
 	  putc_unlocked (':', stream);
 	  putc_unlocked (' ', stream);
@@ -1820,7 +1830,7 @@ __argp_failure (const struct argp_state *state, int status, int errnum,
 	{
 	  __flockfile (stream);
 
-	  fputs_unlocked (short_program_name(state),
+	  fputs_unlocked (__argp_short_program_name(state),
 			  stream);
 
 	  if (fmt)
