@@ -1,6 +1,5 @@
 /* channel_commands.c
  *
- * $Id: channel_commands.c,v 1.26.2.1 2003/09/19 12:15:37 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -57,7 +56,6 @@ do_channel_open_command(struct command *s,
 		      make_channel_open_exception(
 			SSH_OPEN_RESOURCE_SHORTAGE,
 			"Allocating a local channel number failed."));
-      return;
     }
 
   channel = NEW_CHANNEL(self, connection, index, &request);
@@ -193,7 +191,7 @@ do_install_fix_global_request_handler(struct command *s,
 }
 
 struct command *
-make_install_fix_global_request_handler(UINT32 name,
+make_install_fix_global_request_handler(uint32_t name,
 					struct global_request *handler)
 {  
   NEW(install_global_request_handler, self);
@@ -237,7 +235,7 @@ do_install_fix_channel_open_handler(struct command *s,
 }
 
 struct command *
-make_install_fix_channel_open_handler(UINT32 name,
+make_install_fix_channel_open_handler(uint32_t name,
 				      struct channel_open *handler)
 {
   NEW(install_channel_open_handler, self);
