@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.141 2002/08/26 20:14:18 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -21,12 +21,30 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <string.h>
+
+#include <fcntl.h>
+#include <signal.h>
+
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include "client.h"
 
 #include "abstract_io.h"
 #include "channel.h"
 #include "channel_commands.h"
 #include "connection.h"
+#include "environ.h"
 #include "format.h"
 #include "interact.h"
 #include "io.h"
@@ -38,19 +56,7 @@
 #include "translate_signal.h"
 #include "werror.h"
 #include "xalloc.h"
-
-#include <assert.h>
-#include <string.h>
-
-#include <signal.h>
-
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
+#include "io.h"
 
 #include "lsh_argp.h"
 
@@ -113,7 +119,7 @@ do_accept_service(struct packet_handler *c,
 }
 
 struct packet_handler *
-make_accept_service_handler(UINT32 service,
+make_accept_service_handler(uint32_t service,
 			    struct command_continuation *c)
 {
   NEW(accept_service_handler, closure);
@@ -155,6 +161,118 @@ make_request_service(int service)
 }
 
 
+/* GABA:
+   (class
+     (name detach_callback)
+     (super lsh_callback)
+     (vars 
+       (channel_flag . int)
+       (fd_flag . int)
+       (exit_status . "int *")))
+*/
+
+/* GABA:
+   (class
+     (name detach_resource)
+     (super resource)
+     (vars
+       (c object detach_callback)))
+*/
+
+static void 
+do_detach_res_kill(struct resource *r)
+{
+  CAST(detach_resource,self,r);
+
+  trace("client.c:do_detach_res\n");
+  self->c->channel_flag = 1;
+
+  if (self->c->channel_flag && self->c->fd_flag)
+    /* If the fd_flag is set, the callback should be changed */
+    io_callout(&self->c->super, 0);
+}
+
+static struct resource*
+make_detach_resource(struct lsh_callback *c)
+{
+   NEW(detach_resource, self);
+   CAST(detach_callback, cb, c);
+
+   trace("client.c:make_detach_resource\n");
+   init_resource(&self->super, do_detach_res_kill);
+
+   self->c = cb;
+
+   return &self->super;
+}
+
+
+static void 
+do_detach_cb(struct lsh_callback *c)
+{
+  CAST(detach_callback,self,c);
+
+  trace("client.c: do_detach_cb\n");
+  
+  if (!self->fd_flag) /* First time around? */
+    {
+      self->fd_flag = 1; /* Note */
+      
+    if (self->channel_flag && self->fd_flag)
+      /* If the fd is closed already, asked to be called from the main loop */ 
+      io_callout(c, 0);
+
+    }
+  else
+    {
+      int pid = fork();
+
+      /* Ignore any errors, what can we do? */
+      
+      switch(pid)
+	{
+	case -1: /* Fork failed, this we can handle by doing nothing */
+	  werror("Fork failed, not detaching.\n");
+	  break;
+	  
+	case 0:
+	  /* Detach, lsh doesn't actually use these fds but dup(2)s them */
+	  
+	  close(STDIN_FILENO); 
+	  close(STDOUT_FILENO); 
+	  close(STDERR_FILENO); 
+	  
+	  /* Make sure they aren't used by any file lsh opens */
+	  
+	  open("/dev/null", O_RDONLY);
+	  open("/dev/null", O_RDONLY);
+	  open("/dev/null", O_RDONLY);
+	  break;
+	  
+	default:
+	  /* It's a good idea to reset std* to blocking mode. */
+	  io_set_blocking(STDIN_FILENO);
+	  io_set_blocking(STDOUT_FILENO);
+	  io_set_blocking(STDERR_FILENO);
+	  
+	  exit(*self->exit_status);
+	}
+    }
+}
+
+static struct lsh_callback* 
+make_detach_callback(int *exit_status)
+{
+   NEW(detach_callback, self);
+
+   self->super.f = do_detach_cb;
+   self->exit_status = exit_status;
+   self->fd_flag = 0;
+   self->channel_flag = 0;
+
+   return &self->super;
+}
+
 /* GABA:
    (class
      (name exit_handler)
@@ -172,7 +290,7 @@ do_exit_status(struct channel_request *c,
 	       struct exception_handler *e)
 {
   CAST(exit_handler, closure, c);
-  UINT32 status;
+  uint32_t status;
 
   if (!info->want_reply
       && parse_uint32(args, &status)
@@ -180,12 +298,11 @@ do_exit_status(struct channel_request *c,
     {
       verbose("client.c: Receiving exit-status %i on channel %i\n",
 	      status, channel->channel_number);
-      
-      *closure->exit_status = status;
 
+      *closure->exit_status = status;
       ALIST_SET(channel->request_types, ATOM_EXIT_STATUS, NULL);
       ALIST_SET(channel->request_types, ATOM_EXIT_SIGNAL, NULL);
-
+      
       COMMAND_RETURN(s, NULL);
     }
   else
@@ -203,14 +320,14 @@ do_exit_signal(struct channel_request *c,
 {
   CAST(exit_handler, closure, c);
 
-  UINT32 signal;
+  uint32_t signal;
   int core;
 
-  const UINT8 *msg;
-  UINT32 length;
+  const uint8_t *msg;
+  uint32_t length;
 
-  const UINT8 *language;
-  UINT32 language_length;
+  const uint8_t *language;
+  uint32_t language_length;
   
   if (!info->want_reply
       && parse_atom(args, &signal)
@@ -275,7 +392,7 @@ make_handle_exit_signal(int *exit_status)
 static struct ssh_channel *
 new_session(struct channel_open_command *s,
 	    struct ssh_connection *connection,
-	    UINT32 local_channel_number,
+	    uint32_t local_channel_number,
 	    struct lsh_string **request)
 {
   CAST(session_open_command, self, s);
@@ -412,6 +529,7 @@ make_client_session(struct client_options *options);
 #define OPT_FORK_STDIO 0x213
 
 #define OPT_SUBSYSTEM 0x214
+#define OPT_DETACH 0x215
 
 void
 init_client_options(struct client_options *self,
@@ -432,7 +550,7 @@ init_client_options(struct client_options *self,
   self->port = NULL;
   self->remote = NULL;
 
-  self->local_user = self->user = getenv("LOGNAME");
+  self->local_user = self->user = getenv(ENV_LOGNAME);
 
   self->with_remote_peers = 0; 
   self->with_pty = -1;
@@ -450,9 +568,12 @@ init_client_options(struct client_options *self,
   self->used_pty = 0;
   self->used_x11 = 0;
   
+  self->detach_end = 0;
   self->start_shell = 1;
   self->remote_forward = 0;
 
+  self->inhibit_actions = 0;
+
   object_queue_init(&self->actions);  
 }
 
@@ -533,6 +654,9 @@ client_options[] =
     "is applied to stderr only.", 0 },
   { "no-cvs-workaround", OPT_FORK_STDIO | ARG_NOT, NULL, 0,
     "Disable the cvs workaround.", 0 },
+  { "detach", OPT_DETACH, NULL, 0, "Detach from terminal at session end.", 0},
+  { "no-detach", OPT_DETACH | ARG_NOT, NULL, 0, "Do not detach session at end," 
+    " wait for all open channels (default).", 0},
 
 #if WITH_PTY_SUPPORT
   { "pty", 't', NULL, 0, "Request a remote pty (default).", 0 },
@@ -614,7 +738,7 @@ client_maybe_x11(struct client_options *options,
 {
   if (options->with_x11)
     {
-      char *display = getenv("DISPLAY");
+      char *display = getenv(ENV_DISPLAY);
       struct command *request = NULL;
       
       assert(options->random);
@@ -888,6 +1012,7 @@ make_client_session(struct client_options *options)
   struct ssh_channel *session;
   
   struct escape_info *escape = NULL;
+  struct lsh_callback *detach_cb = NULL;
   
   debug("lsh.c: Setting up stdin\n");
 
@@ -975,6 +1100,9 @@ make_client_session(struct client_options *options)
       return NULL;
     }
 
+  if (options->detach_end) /* Detach? */
+    detach_cb = make_detach_callback(options->exit_code);  
+
   /* Clear options */
   options->stdin_file = options->stdout_file = options->stderr_file = NULL;
 
@@ -982,12 +1110,20 @@ make_client_session(struct client_options *options)
     (io_read(make_lsh_fd(in, "client stdin", options->handler),
 	     NULL, NULL),
      io_write(make_lsh_fd(out, "client stdout", options->handler),
-	      BLOCK_SIZE, NULL),
+	      BLOCK_SIZE, 
+	      detach_cb
+	      ),
      io_write(make_lsh_fd(err, "client stderr", options->handler),
 	      BLOCK_SIZE, NULL),
      escape,
      WINDOW_SIZE,
      options->exit_code);
+  
+  if (options->detach_end)
+    {
+      remember_resource(session->resources, make_detach_resource(detach_cb));
+      options->detach_end = 0;
+    }
 
   /* The channel won't get registered in any other resource_list
    * until later, so we must register it here to avoid a "garbage
@@ -998,10 +1134,64 @@ make_client_session(struct client_options *options)
 }
 
 
+/* Treat environment variables as sources for options */
+
+void
+envp_parse(const struct argp *argp,
+	   const char** envp,
+	   const char* name,
+	   unsigned flags, 
+	   void *input)
+{
+  CAST_SUBTYPE(client_options, options, input);
+  int nlen = strlen(name);
+
+  while (*envp)
+    {
+      if (!strncmp(*envp, name, nlen)) 	  /* Matching environment entry */
+	{
+	  char** sim_argv;
+	  char* entry;
+
+	  /* Make a copy we can modify */
+	  
+	  entry = strdup(*envp+nlen); 	  /* Skip variable name */
+
+	  if (entry)
+	    {
+	      /* Extra space doesn't hurt */
+	      sim_argv = malloc(sizeof(char*) * (strlen(entry)+2));
+
+	      if (sim_argv)
+		{
+		  int sim_argc = 1;
+		  char *token = strtok(entry, " \n\t");
+		  
+		  sim_argv[0] = "";
+
+		  while (token) /* For all tokens in variable */
+		    {
+		      sim_argv[sim_argc++] = token;
+		      token = strtok( NULL, " \n\t");
+		    }
+
+		  sim_argv[sim_argc] = NULL;
+	
+		  options->inhibit_actions = 1; /* Disable nnormal actions performed at end */
+		  argp_parse(argp, sim_argc, sim_argv, flags | ARGP_NO_ERRS | ARGP_NO_EXIT, NULL, input);
+		  options->inhibit_actions = 0; /* Reenable */
+		}
+	    }
+	}
+
+      envp++; 
+   }
+}
+
 /* Parse the argument for -R and -L */
 int
 client_parse_forward_arg(char *arg,
-			 UINT32 *listen_port,
+			 uint32_t *listen_port,
 			 struct address_info **target)
 {
   char *first;
@@ -1106,6 +1296,9 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
       break;
 
     case ARGP_KEY_END:
+      if (options->inhibit_actions)
+	break;
+
       if (!options->user)
 	{
 	  argp_error(state, "No user name given. Use the -l option, or set LOGNAME in the environment.");
@@ -1174,7 +1367,7 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 
     case 'L':
       {
-	UINT32 listen_port;
+	uint32_t listen_port;
 	struct address_info *target;
 
 	if (!client_parse_forward_arg(arg, &listen_port, &target))
@@ -1203,7 +1396,9 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 #if WITH_PTY_SUPPORT
     CASE_FLAG('t', with_pty);
 #endif /* WITH_PTY_SUPPORT */
-    
+
+    CASE_FLAG(OPT_DETACH, detach_end);
+
     CASE_ARG(OPT_STDIN, stdin_file, "/dev/null");
     CASE_ARG(OPT_STDOUT, stdout_file, "/dev/null"); 
     CASE_ARG(OPT_STDERR, stderr_file, "/dev/null");
