@@ -1,8 +1,6 @@
 /* client.h
  *
- *
- *
- * $Id: client.h,v 1.49 2001/12/12 18:37:54 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -39,7 +37,7 @@
 #define CLIENT_ARGP_MODIFIER_GROUP 200
 
 struct packet_handler *
-make_accept_service_handler(UINT32 service,
+make_accept_service_handler(uint32_t service,
 			    struct command_continuation *c);
 
 /* GABA:
@@ -65,12 +63,12 @@ struct command *make_request_service(int service);
    (class
      (name escape_info)
      (vars
-       (escape . UINT8)
+       (escape . uint8_t)
        ; Handlers, indexed by character.
        (dispatch array (object lsh_callback) "0x100")))
 */
 
-struct escape_info *make_escape_info(UINT8 escape);
+struct escape_info *make_escape_info(uint8_t escape);
 struct abstract_write *
 make_handle_escape(struct escape_info *info, struct abstract_write *next);
 
@@ -91,7 +89,7 @@ make_client_session_channel(struct lsh_fd *in,
 			    struct lsh_fd *out,
 			    struct lsh_fd *err,
 			    struct escape_info *escape,
-			    UINT32 initial_window,
+			    uint32_t initial_window,
 			    int *exit_status);
 
 struct command *
@@ -159,6 +157,12 @@ make_client_x11_display(const char *display, struct lsh_string *fake);
        (used_pty . int)
        (used_x11 . int)
        
+       ; True if the client should detach when a session closes (useful for gateways)
+       (detach_end . int)
+
+       ; Inhibit actions, used to not create actions from environment parsing.
+       (inhibit_actions . int)
+
        (start_shell . int)
        (remote_forward . int)
        (actions struct object_queue)))
@@ -173,10 +177,9 @@ init_client_options(struct client_options *options,
 struct command *
 client_add_action(struct client_options *options,
 		  struct command *action);
-
 int
 client_parse_forward_arg(char *arg,
-			 UINT32 *listen_port,
+			 uint32_t *listen_port,
 			 struct address_info **target);
 
 extern struct command client_options2remote;
@@ -187,4 +190,12 @@ extern struct command client_options2actions;
 
 extern const struct argp client_argp;
 
+
+void
+envp_parse(const struct argp *argp,
+           const char** envp,
+           const char* name,
+           unsigned flags,
+           void *input);
+
 #endif /* LSH_CLIENT_H_INCLUDED */
