@@ -1,6 +1,5 @@
 /* connection_commands.c
  *
- * $Id: connection_commands.c,v 1.40 2001/08/21 10:56:20 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -47,7 +46,7 @@ DEFINE_COMMAND2(connection_remember)
   CAST_SUBTYPE(resource, resource, a2);
   
   if (resource)
-    REMEMBER_RESOURCE(connection->resources, resource);
+    remember_resource(connection->resources, resource);
 
   COMMAND_RETURN(c, resource);
 }
