@@ -1,10 +1,10 @@
 /* exception.h
  *
- * $Id: exception.h,v 1.26 2002/06/27 17:08:20 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
- * Copyright (C) 1998 Niels Mller
+ * Copyright (C) 1998, 2002 Niels Mller
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,7 +34,7 @@
    (class
      (name exception)
      (vars
-       (type . UINT32)
+       (type . uint32_t)
        (msg . "const char *")))
 */
 
@@ -128,16 +128,6 @@ void exception_raise(struct exception_handler *e,
 /* DNS errors */
 #define EXC_RESOLVE 0x20001
 
-/* SEXP parse errors */
-#define EXC_SEXP 0x40000
-#define EXC_SEXP_SYNTAX 0x40001
-#define EXC_SEXP_EOF 0x40002
-
-/* SPKI errors */
-#define EXC_SPKI 0x80000
-/* Unexpected or unsupported expression type */
-#define EXC_SPKI_TYPE 0x80001
-
 /* Pausing, used to temporarily stop reading packets on a connection. */
 #define EXC_PAUSE 0x100000
 
@@ -161,13 +151,13 @@ extern struct exception_handler ignore_exception_handler;
    (class
      (name report_exception_info)
      (vars
-       (mask . UINT32)
-       (value . UINT32)
+       (mask . uint32_t)
+       (value . uint32_t)
        (prefix . "const char *")))
 */
 
 struct report_exception_info *
-make_report_exception_info(UINT32 mask, UINT32 value,
+make_report_exception_info(uint32_t mask, uint32_t value,
 			   const char *prefix);
 
 #define STATIC_REPORT_EXCEPTION_INFO(m, v, p) \
@@ -179,7 +169,7 @@ make_report_exception_handler(const struct report_exception_info *info,
 			      const char *context);
 
 struct exception *
-make_simple_exception(UINT32 type, const char *msg);
+make_simple_exception(uint32_t type, const char *msg);
 
 /* Create a simple exception handler, with no internal state */
 struct exception_handler *
@@ -196,13 +186,13 @@ make_exception_handler(void (*raise)(struct exception_handler *s,
      (vars
        ; A reason code that can be passed in a SSH_MSG_DISCONNECT message.
        ; Zero means terminate the connection without sending such a message.
-       (reason . UINT32)))
+       (reason . uint32_t)))
 */
 
 
 /* If msg is NULL, it is derived from the reason value */
 struct exception *
-make_protocol_exception(UINT32 reason, const char *msg);
+make_protocol_exception(uint32_t reason, const char *msg);
 
 #define STATIC_PROTOCOL_EXCEPTION(reason, msg) \
 { { STATIC_HEADER, EXC_PROTOCOL, (msg) }, (reason) }
