@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.175 2002/08/27 16:45:00 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -21,21 +21,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "io.h"
-
-#include "command.h"
-#include "format.h"
-#include "string_buffer.h"
-#include "werror.h"
-#include "xalloc.h"
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <assert.h>
+#include <errno.h>
 #include <string.h>
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
+/* For the popen code */
+#include <wait.h>
+
 #ifdef HAVE_POLL
 # if HAVE_POLL_H
 #  include <poll.h>
@@ -46,8 +46,25 @@
 # include "jpoll.h"
 #endif
 
+#include <fcntl.h>
+#include <signal.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/stat.h>
+#include <sys/un.h>
+
+#include <arpa/inet.h>
+
 #include <oop.h>
 
+#include "io.h"
+
+#include "command.h"
+#include "format.h"
+#include "string_buffer.h"
+#include "werror.h"
+#include "xalloc.h"
+
 /* Workaround for some version of FreeBSD. */
 #ifdef POLLRDNORM
 # define MY_POLLIN (POLLIN | POLLRDNORM)
@@ -55,15 +72,6 @@
 # define MY_POLLIN POLLIN
 #endif /* !POLLRDNORM */
 
-#include <errno.h>
-#include <fcntl.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-#include <arpa/inet.h>
-#include <signal.h>
-#include <sys/stat.h>
-
 
 #define GABA_DEFINE
 #include "io.h.x"
@@ -432,16 +440,11 @@ do_buffered_read(struct io_callback *s,
 		 struct lsh_fd *fd)
 {
   CAST(io_buffered_read, self, s);
-  UINT8 *buffer = alloca(self->buffer_size);
+  uint8_t *buffer = alloca(self->buffer_size);
   int res;
 
   assert(fd->want_read);   
 
-#if 0
-  /* If hanged_up is set, pretend that read returned 0 */
-  res = fd->hanged_up ? 0 : read(fd->fd, buffer, self->buffer_size);
-#endif
-
   res = read(fd->fd, buffer, self->buffer_size);
   
   if (res < 0)
@@ -467,11 +470,11 @@ do_buffered_read(struct io_callback *s,
       }
   else if (res > 0)
     {
-      UINT32 left = res;
+      uint32_t left = res;
     
       while (fd->super.alive && fd->read && left)
 	{
-	  UINT32 done;
+	  uint32_t done;
 
 	  /* NOTE: What to do if want_read is false? To improve the
 	   * connection_lock mechanism, it must be possible to
@@ -532,7 +535,7 @@ do_buffered_read(struct io_callback *s,
 }
 
 struct io_callback *
-make_buffered_read(UINT32 buffer_size,
+make_buffered_read(uint32_t buffer_size,
 		   struct read_handler *handler)
 {
   NEW(io_buffered_read, self);
@@ -549,7 +552,7 @@ do_consuming_read(struct io_callback *c,
 		  struct lsh_fd *fd)
 {
   CAST_SUBTYPE(io_consuming_read, self, c);
-  UINT32 wanted = READ_QUERY(self);
+  uint32_t wanted = READ_QUERY(self);
 
   assert(fd->want_read);
   
@@ -629,7 +632,7 @@ do_write_callback(struct io_callback *s UNUSED,
     }
   else
     {
-      UINT32 size;
+      uint32_t size;
       int res;
 
       size = MIN(fd->write_buffer->end - fd->write_buffer->start,
@@ -834,7 +837,7 @@ do_exc_io_handler(struct exception_handler *self,
 
 /* For fd:s in blocking mode. */
 const struct exception *
-write_raw(int fd, UINT32 length, const UINT8 *data)
+write_raw(int fd, uint32_t length, const uint8_t *data)
 {
   while(length)
     {
@@ -858,7 +861,7 @@ write_raw(int fd, UINT32 length, const UINT8 *data)
 }
 
 const struct exception *
-write_raw_with_poll(int fd, UINT32 length, const UINT8 *data)
+write_raw_with_poll(int fd, uint32_t length, const uint8_t *data)
 {
   while(length)
     {
@@ -903,7 +906,7 @@ write_raw_with_poll(int fd, UINT32 length, const UINT8 *data)
 
 /* For fd:s in blocking mode. */
 const struct exception *
-read_raw(int fd, UINT32 length, UINT8 *data)
+read_raw(int fd, uint32_t length, uint8_t *data)
 {
   while(length)
     {
@@ -934,7 +937,7 @@ read_raw(int fd, UINT32 length, UINT8 *data)
 }
 
 struct lsh_string *
-io_read_file_raw(int fd, UINT32 guess)
+io_read_file_raw(int fd, uint32_t guess)
 {
   struct string_buffer buffer;
   string_buffer_init(&buffer, guess);
@@ -1029,7 +1032,7 @@ make_address_info_c(const char *host,
 }
 
 struct address_info *
-make_address_info(struct lsh_string *host, UINT32 port)
+make_address_info(struct lsh_string *host, uint32_t port)
 {
   NEW(address_info, info);
 
@@ -1039,7 +1042,7 @@ make_address_info(struct lsh_string *host, UINT32 port)
 }
 
 struct address_info *
-sockaddr2info(size_t addr_len UNUSED,
+sockaddr2info(size_t addr_len,
 	      struct sockaddr *addr)
 {
   NEW(address_info, info);
@@ -1047,35 +1050,40 @@ sockaddr2info(size_t addr_len UNUSED,
   switch(addr->sa_family)
     {
     case AF_INET:
+      assert(addr_len == sizeof(struct sockaddr_in));
       {
 	struct sockaddr_in *in = (struct sockaddr_in *) addr;
-	UINT32 ip = ntohl(in->sin_addr.s_addr);
+	uint32_t ip = ntohl(in->sin_addr.s_addr);
+	
 	info->port = ntohs(in->sin_port);
 	info->ip = ssh_format("%di.%di.%di.%di",
 			      (ip >> 24) & 0xff,
 			      (ip >> 16) & 0xff,
 			      (ip >> 8) & 0xff,
 			      ip & 0xff);
-	return info;
       }
+      return info;
+      
 #if WITH_IPV6
     case AF_INET6:
+      assert(addr_len == sizeof(struct sockaddr_in6));
       {
 	struct sockaddr_in6 *in = (struct sockaddr_in6 *) addr;
-	UINT8 *ip = in->sin6_addr.s6_addr;
+
 	info->port = ntohs(in->sin6_port);
-	info->ip = ssh_format("%xi:%xi:%xi:%xi:%xi:%xi:%xi:%xi",
-			      (ip[0]  << 8) | ip[1],
-			      (ip[2]  << 8) | ip[3],
-			      (ip[4]  << 8) | ip[5],
-			      (ip[6]  << 8) | ip[7],
-			      (ip[8]  << 8) | ip[9],
-			      (ip[10] << 8) | ip[11],
-			      (ip[12] << 8) | ip[13],
-			      (ip[14] << 8) | ip[15]);
-	return info;
+	info->ip = lsh_string_alloc(INET6_ADDRSTRLEN + 1);
+
+	/* Does inet_ntop always use lower case letters? If not, we
+	 * should perhaps lowercase the result explicitly. */
+	if (!inet_ntop(addr->sa_family, &in->sin6_addr,
+		       info->ip->data, info->ip->length))
+	  fatal("inet_ntop failed for IPv6 address.\n");
+
+	lsh_string_trunc(info->ip, strlen(info->ip->data));
       }
-#endif
+      return info;
+#endif /* WITH_IPV6 */
+
     case AF_UNIX:
       /* Silently return NULL. This happens when a gateway client
        * connects. */
@@ -1742,7 +1750,7 @@ io_connect_local(struct local_info *info,
 struct lsh_fd *
 io_read_write(struct lsh_fd *fd,
 	      struct io_callback *read,
-	      UINT32 block_size,
+	      uint32_t block_size,
 	      struct lsh_callback *close_callback)
 {
   trace("io.c: Preparing fd %i for reading and writing\n",
@@ -1782,7 +1790,7 @@ io_read(struct lsh_fd *fd,
 
 struct lsh_fd *
 io_write(struct lsh_fd *fd,
-	 UINT32 block_size,
+	 uint32_t block_size,
 	 struct lsh_callback *close_callback)
 {
   trace("io.c: Preparing fd %i for writing\n", fd->fd);
@@ -1798,7 +1806,7 @@ io_write(struct lsh_fd *fd,
 
 struct lsh_fd *
 io_write_file(const char *fname, int flags, int mode,
-	      UINT32 block_size,
+	      uint32_t block_size,
 	      struct lsh_callback *c,
 	      struct exception_handler *e)
 {
@@ -1997,7 +2005,7 @@ const struct exception finish_io_exception =
 STATIC_EXCEPTION(EXC_FINISH_IO, "Stop i/o");
 
 struct exception *
-make_io_exception(UINT32 type, struct lsh_fd *fd, int error, const char *msg)
+make_io_exception(uint32_t type, struct lsh_fd *fd, int error, const char *msg)
 {
   NEW(io_exception, self);
   assert(type & EXC_IO);
@@ -2052,6 +2060,99 @@ lsh_make_pipe(int *fds)
   return 1;
 }
 
+/* Forks a filtering process, and reads the output. */
+int
+lsh_popen(const char *program, const char **argv, int in,
+	  pid_t *child)
+{
+  /* 0 for read, 1, for write */
+  int out[2];
+  pid_t pid;
+  
+  if (!lsh_make_pipe(out))
+    {
+      close(in);
+      return -1;
+    }
+  pid = fork();
+
+  if (pid < 0)
+    {
+      close(in);
+      close(out[0]);
+      close(out[1]);
+      return -1;
+    }
+  else if (!pid)
+    {
+      /* Child */
+      if (dup2(in, STDIN_FILENO) < 0)
+	{
+	  werror("lsh_popen: dup2 for stdin failed %e.\n", errno);
+	  _exit(EXIT_FAILURE);
+	}
+      if (dup2(out[1], STDOUT_FILENO) < 0)
+	{
+	  werror("lsh_popen: dup2 for stdout failed %e.\n", errno);
+	  _exit(EXIT_FAILURE);
+	}
+
+      close(in);
+      close(out[0]);
+      close(out[1]);
+
+      /* The execv prototype uses const in the wrong way */
+      execv(program, (char **) argv);
+
+      werror("lsh_popen: execv `%z' failed %e.\n", program, errno);
+
+      _exit(EXIT_FAILURE);
+    }
+  else
+    {
+      /* Parent process */
+      close(in);
+      close(out[1]);
+      *child = pid;
+      return out[0];
+    }
+}
+
+struct lsh_string *
+lsh_popen_read(const char *program, const char **argv, int in,
+	       unsigned guess)
+{
+  pid_t pid;
+  int status;
+  int fd = lsh_popen(program, argv, in, &pid);
+  struct lsh_string *s = io_read_file_raw(fd, guess);
+  
+  if (waitpid(pid, &status, 0) < 0)
+    {
+      werror("lsh_popen_read: waitpid failed: %e\n", errno);
+      lsh_string_free(s);
+      return NULL;
+    }
+
+  if (!s)
+    return s;
+
+  if (WIFEXITED(status))
+    {
+      if (!WEXITSTATUS(status))
+	/* Success. */
+	return s;
+
+      werror("sexp-conv exited with status %i.\n", WEXITSTATUS(status));
+    }
+  else
+    werror("sexp-conv terminated by signal %i (%z).\n",
+	   WTERMSIG(status), strsignal(WTERMSIG(status)));
+
+  lsh_string_free(s);
+  return NULL;
+}
+
 /* Copies data from one fd to another. Works no matter if the fd:s are
  * in blocking or non-blocking mode. Tries hard not to do premature
  * reads; we don't want to read data into the buffer, and then
@@ -2074,7 +2175,7 @@ lsh_copy_file(int src, int dst)
   for (;;)
     {
       int res;
-      UINT32 i, length;
+      uint32_t i, length;
 
       /* First wait until dst is writable; otherwise there's no point
        * in reading the input. */
