@@ -1,6 +1,5 @@
 /* io_commands.h
  *
- * $Id: io_commands.h,v 1.28 2002/08/26 20:16:57 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -39,11 +38,11 @@
        (name string)
        (flags . int)
        (mode . int)
-       (block_size . UINT32)))
+       (block_size . uint32_t)))
 */
 
 struct io_write_file_info *
-make_io_write_file_info(struct lsh_string *name, int flags, int mode, UINT32 block_size);
+make_io_write_file_info(struct lsh_string *name, int flags, int mode, uint32_t block_size);
 
 extern struct command io_write_file_command;
 
@@ -68,13 +67,6 @@ extern struct command_2 connect_connection_command;
 extern struct command connect_simple_command;
 #define CONNECT_SIMPLE (&connect_simple_command.super)
 
-#if 0
-struct command *
-make_listen_local(struct local_info *info);
-
-struct command *
-make_connect_local(void);
-#endif
 
 extern struct command connect_local_command;
 #define CONNECT_LOCAL (&connect_local_command.super)
