@@ -1,6 +1,6 @@
 /* keyexchange.c
  *
- * $Id: keyexchange.c,v 1.86 2002/09/12 19:43:53 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -21,6 +21,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <string.h>
+
 #include "keyexchange.h"
 
 #include "abstract_io.h"
@@ -38,9 +45,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <string.h>
-#include <assert.h>
-
 #define GABA_DEFINE
 #include "keyexchange.h.x"
 #undef GABA_DEFINE
@@ -71,15 +75,18 @@
 #define NLISTS 10
 
 /* Arbitrary limit on list length */
+/* An SSH-2.0-Sun_SSH_1.0 server has been reported to list 250
+ * different algorithms, or in fact a list of installed locales. */
 #define KEXINIT_MAX_ALGORITMS 47
+#define KEXINIT_MAX_ALGORITMS_SUN 300
 
 static struct kexinit *
-parse_kexinit(struct lsh_string *packet)
+parse_kexinit(struct lsh_string *packet, enum peer_flag peer_flags)
 {
   NEW(kexinit, res);
   struct simple_buffer buffer;
   unsigned msg_number;
-  UINT32 reserved;
+  uint32_t reserved;
   
   struct int_list *lists[NLISTS];
   int i;
@@ -101,7 +108,13 @@ parse_kexinit(struct lsh_string *packet)
   
   for (i = 0; i<NLISTS; i++)
     {
-      if ( !(lists[i] = parse_atom_list(&buffer, KEXINIT_MAX_ALGORITMS)))
+      unsigned limit = KEXINIT_MAX_ALGORITMS;
+#if DATAFELLOWS_WORKAROUNDS
+      if ( (peer_flags & PEER_KEXINIT_LANGUAGE_KLUDGE)
+	   && i >= 8)
+	limit = KEXINIT_MAX_ALGORITMS_SUN;
+#endif /* DATAFELLOWS_WORKAROUNDS */
+      if ( !(lists[i] = parse_atom_list(&buffer, limit)))
 	break;
     }
 
@@ -238,7 +251,7 @@ do_handle_kexinit(struct packet_handler *c,
   struct object_list *algorithms;
 
   int mode = connection->flags & CONNECTION_MODE;
-  struct kexinit *msg = parse_kexinit(packet);
+  struct kexinit *msg = parse_kexinit(packet, connection->peer_flags);
 
   int i;
 
@@ -381,16 +394,16 @@ make_kexinit_handler(struct lsh_object *extra,
 
 static struct lsh_string *
 kex_make_key(struct hash_instance *secret,
-	     UINT32 key_length,
+	     uint32_t key_length,
 	     int type,
 	     struct lsh_string *session_id)
 {
   /* Indexed by the KEX_* values */
-  static const UINT8 tags[] = "CDEFAB";
+  static const uint8_t tags[] = "CDEFAB";
   
   struct lsh_string *key;
   struct hash_instance *hash;
-  UINT8 *digest;
+  uint8_t *digest;
   
   key = lsh_string_alloc(key_length);
 
@@ -413,7 +426,7 @@ kex_make_key(struct hash_instance *secret,
   else
     {
       unsigned left = key_length;
-      UINT8 *dst = key->data;
+      uint8_t *dst = key->data;
       
       KILL(hash);
       hash = hash_copy(secret);
