@@ -2,7 +2,7 @@
  *
  * Reads a (private) key on stdin, and saves it a private and a public file.
  *
- * $Id: lsh-writekey.c,v 1.26 2002/08/26 20:14:18 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -23,30 +23,36 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <errno.h>
+
+#include <fcntl.h>
+
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <sys/types.h>
+#include <sys/stat.h>
+
+
 #include "algorithms.h"
 #include "crypto.h"
-#include "dsa.h"
+#include "environ.h"
 #include "format.h"
 #include "io_commands.h"
 #include "interact.h"
 #include "publickey_crypto.h"
-#include "sexp.h"
 #include "spki.h"
 #include "version.h"
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-#include <errno.h>
-/* #include <string.h> */
-
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
+#include "nettle/sexp.h"
 
 #include "lsh-writekey.c.x"
 
@@ -71,7 +77,6 @@ const char *argp_program_bug_address = BUG_ADDRESS;
        (tty object interact)
        
        (label string)
-       (style . sexp_argp_state)
        (passphrase string)
 
        (crypto_algorithms object alist)
@@ -81,7 +86,7 @@ const char *argp_program_bug_address = BUG_ADDRESS;
        
        (crypto_name . int)
        (crypto object crypto_algorithm)
-       (iterations . UINT32)))
+       (iterations . uint32_t)))
 */
 
 static struct lsh_writekey_options *
@@ -96,7 +101,6 @@ make_lsh_writekey_options(void)
   self->tty = make_unix_interact();
     
   self->label = NULL;
-  self->style = -1;
 
   self->passphrase = NULL;
   self->iterations = 1500;
@@ -133,7 +137,6 @@ main_options[] =
 static const struct argp_child
 main_argp_children[] =
 {
-  { &sexp_output_argp, 0, NULL, 0 },
   { &werror_argp, 0, "", 0 },
   { NULL, 0, NULL, 0}
 };
@@ -150,8 +153,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       return ARGP_ERR_UNKNOWN;
 
     case ARGP_KEY_INIT:
-      state->child_inputs[0] = &self->style;
-      state->child_inputs[1] = NULL;
+      state->child_inputs[0] = NULL;
       break;
 
     case ARGP_KEY_END:
@@ -161,7 +163,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	    self->private_file = make_string("/etc/lsh_host_key");
 	  else
 	    {
-	      char *home = getenv("HOME");
+	      char *home = getenv(ENV_HOME);
 	      struct lsh_string *s;
 	  
 	      if (!home)
@@ -188,7 +190,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	{
 	  if (!self->label)
 	    {
-	      const char *name = getenv("LOGNAME");
+	      const char *name = getenv(ENV_LOGNAME);
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 300
 #endif
@@ -233,7 +235,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	    }
 	  self->r = (self->server
 		     ? make_system_random()
-		     : make_user_random(getenv("HOME")));
+		     : make_user_random(getenv(ENV_HOME)));
 	  if (!self->r)
 	    argp_failure(state, EXIT_FAILURE, 0, 
 			 "Failed to initialize randomness generator.");
@@ -346,42 +348,38 @@ open_file(const struct lsh_string *file)
 }
 
 static struct lsh_string *
-process_private(struct sexp *key,
+process_private(struct lsh_string *key,
                 struct lsh_writekey_options *options)
 {
-  struct sexp *expr = key;
-
   if (options->crypto)
     {
       CAST_SUBTYPE(mac_algorithm, hmac,
                    ALIST_GET(options->crypto_algorithms, ATOM_HMAC_SHA1));
       assert(hmac);
       
-      expr = spki_pkcs5_encrypt(options->r,
-                                options->label,
-                                ATOM_HMAC_SHA1,
-                                hmac,
-                                options->crypto_name,
-                                options->crypto,
-                                10, /* Salt length */
-                                options->passphrase,
-                                options->iterations,
-                                sexp_format(key, SEXP_CANONICAL, 0));
+      return spki_pkcs5_encrypt(options->r,
+				options->label,
+				ATOM_HMAC_SHA1,
+				hmac,
+				options->crypto_name,
+				options->crypto,
+				10, /* Salt length */
+				options->passphrase,
+				options->iterations,
+				key);
     }
-  return sexp_format(expr,
-                     (options->style > 0) ? options->style : SEXP_CANONICAL,
-                     0);
+  else
+    return lsh_string_dup(key);
 }
 
 static struct lsh_string *
-process_public(struct sexp *key,
+process_public(struct lsh_string *key,
                struct lsh_writekey_options *options)
 {
   struct signer *s;
   struct verifier *v;
   
-  s = spki_sexp_to_signer(options->signature_algorithms,
-                          key, NULL);
+  s = spki_make_signer(options->signature_algorithms, key, NULL);
   
   if (!s)
     return NULL;
@@ -389,9 +387,7 @@ process_public(struct sexp *key,
   v = SIGNER_GET_VERIFIER(s);
   assert(v);
 
-  return sexp_format(spki_make_public_key(v),
-                     (options->style > 0) ? options->style : SEXP_TRANSPORT,
-                     0);
+  return PUBLIC_SPKI_KEY(v, 1);
 }
 
 int
@@ -402,7 +398,6 @@ main(int argc, char **argv)
   int public_fd;
   struct lsh_string *input;
   struct lsh_string *output;
-  struct sexp *key;
   const struct exception *e;
   
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
@@ -419,15 +414,7 @@ main(int argc, char **argv)
       return EXIT_FAILURE;
     }
   
-  key = string_to_sexp(SEXP_TRANSPORT, input, 1);
-
-  if (!key)
-    {
-      werror("S-expression syntax error.\n");
-      return EXIT_FAILURE;
-    }
-
-  output = process_private(key, options);
+  output = process_private(input, options);
   if (!output)
     return EXIT_FAILURE;
 
@@ -445,7 +432,9 @@ main(int argc, char **argv)
       return EXIT_FAILURE;
     }
 
-  output = process_public(key, options);
+  output = process_public(input, options);
+  lsh_string_free(input);
+  
   if (!output)
     return EXIT_FAILURE;
 
