@@ -1,8 +1,8 @@
 /* lsh.c
  *
- * client main program
+ * Client main program.
  *
- * $Id: lsh.c,v 1.167 2002/08/29 21:51:11 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -23,6 +23,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <errno.h>
+#include <locale.h>
+#include <stdio.h>
+#include <stdlib.h>
+
+#include <fcntl.h>
+
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include "algorithms.h"
 #include "alist.h"
 #include "atoms.h"
@@ -34,7 +49,7 @@
 #include "compress.h"
 #include "connection_commands.h"
 #include "crypto.h"
-#include "dsa.h"
+#include "environ.h"
 #include "format.h"
 #include "interact.h"
 #include "io.h"
@@ -54,16 +69,10 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-#include <errno.h>
-#include <locale.h>
-#include <stdio.h>
-#include <stdlib.h>
-/* #include <string.h> */
+#include "nettle/sexp.h"
+/* For struct spki_iterator */
+#include "spki/parse.h"
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 
 #include "lsh_argp.h"
 
@@ -132,7 +141,7 @@ make_options(struct exception_handler *handler,
 	     int *exit_code)
 {
   NEW(lsh_options, self);
-  const char *home = getenv("HOME");
+  const char *home = getenv(ENV_HOME);
   struct randomness *r = make_user_random(home);
 
   init_client_options(&self->super, 
@@ -142,7 +151,8 @@ make_options(struct exception_handler *handler,
   self->algorithms = make_algorithms_options(all_symmetric_algorithms());
   self->home = home;
 
-  self->signature_algorithms = all_signature_algorithms(r); /* OK to init with NULL */
+  /* OK to init with NULL */  
+  self->signature_algorithms = all_signature_algorithms(r);
 
   self->sloppy = 0;
   self->capture = NULL;
@@ -177,65 +187,89 @@ DEFINE_COMMAND(options2service)
   COMMAND_RETURN(c, options->service);
 }
 
-/* Open hostkey database. By default, "~/.lsh/known_hosts". */
+/* Open hostkey database. By default, "~/.lsh/host-acls". */
 
 static struct spki_context *
 read_known_hosts(struct lsh_options *options)
 {
   struct lsh_string *tmp = NULL;
+  const char *s;
   struct lsh_string *contents;
-  const char *s = NULL;
   int fd;
-  struct simple_buffer buffer;
+  struct spki_iterator i;
   struct spki_context *context;
-
+  const char *sexp_conv = getenv(ENV_SEXP_CONV);
+  const char *args[] = { "sexp-conv", "-s", "canonical", NULL };
+  
   context = make_spki_context(options->signature_algorithms);
   
   if (options->known_hosts)
-    s = options->known_hosts;
+    {
+      s = options->known_hosts;
+      fd = open(s, O_RDONLY);
+    }
   else
     {
-      tmp = ssh_format("%lz/.lsh/known_hosts", options->home);
+      tmp = ssh_format("%lz/.lsh/host-acls", options->home);
       s = lsh_get_cstring(tmp);
+      fd = open(s, O_RDONLY);
+      
+      if (fd < 0)
+	{
+	  struct stat sbuf;
+
+	  /* FIXME: Don't reuse tmp here. */
+	  lsh_string_free(tmp);
+	  tmp = ssh_format("%lz/.lsh/known_hosts", options->home);
+
+	  if (stat(lsh_get_cstring(tmp), &sbuf) == 0)
+	    {
+	      werror("You have an old known-hosts file `%S'.\n"
+		     "To work with lsh-2.0, run the lsh-upgrade script,\n"
+		     "which will convert that to a new host-acls file.\n",
+		     tmp);
+	    }
+	}
     }
 
-  fd = open(s, O_RDONLY);
   if (fd < 0)
     {
-      lsh_string_free(tmp);
       werror("Failed to open `%z' for reading %e\n", s, errno);
+      lsh_string_free(tmp);
+
       return context;
     }
 
   lsh_string_free(tmp);
 
-  contents = io_read_file_raw(fd, 5000);
+  if (!sexp_conv)
+    sexp_conv = PATH_SEXP_CONV;
+  
+  contents = lsh_popen_read(sexp_conv, args, fd, 5000);
   
   if (!contents)
   {
-    werror("Failed to read known_hosts file %e\n", errno);
+    werror("Failed to read host-acls file %e\n", errno);
     close(fd);
     return context;
   }
 
   close(fd);
 
-  simple_buffer_init(&buffer, contents->length, contents->data);
-  while (!parse_eod(&buffer))
-    {
-      struct sexp *expr = sexp_parse(SEXP_TRANSPORT, &buffer);
-
-      if (!expr)
-        {
-          werror("read_known_hosts: S-expression syntax error.\n");
-          break;
-        }
-      if (!spki_add_acl(context, expr))
-        {
-          werror("read_known_hosts: Invalid ACL.\n");
-          break;
-        }
-    }
+  /* We could use transport syntax instead. That would have the
+   * advantage that we can read and process one entry at a time. */
+  if (!spki_iterator_first(&i, contents->length, contents->data))
+    werror("read_known_hosts: S-expression syntax error.\n");
+    
+  else
+    while (i.type != SPKI_TYPE_END_OF_EXPR)
+      {
+	if (!spki_add_acl(context, &i))
+	  {
+	    werror("read_known_hosts: Invalid ACL.\n");
+	    break;
+	  }
+      }
   lsh_string_free(contents);
   return context;
 }
@@ -247,7 +281,6 @@ read_user_keys(struct lsh_options *options)
   struct lsh_string *tmp = NULL;
   struct lsh_string *contents;
   const char *name = NULL;
-  struct sexp *expr;
   int fd;
   int algorithm_name;
   
@@ -291,14 +324,6 @@ read_user_keys(struct lsh_options *options)
 
   close(fd);
 
-  expr = string_to_sexp(SEXP_TRANSPORT, contents, 1);
-
-  if (!expr)
-    {
-      werror("S-expression syntax error in private key.\n");
-      return make_object_list(0, -1);
-    }
-
   if (options->super.tty)
     {
       struct alist *mac = make_alist(0, -1);
@@ -310,18 +335,21 @@ read_user_keys(struct lsh_options *options)
 		     4, ATOM_3DES_CBC, ATOM_BLOWFISH_CBC,
 		     ATOM_TWOFISH_CBC, ATOM_AES256_CBC, -1);
       
-      expr = spki_pkcs5_decrypt(mac, crypto,
-                                options->super.tty,
-                                expr);
-      if (!expr)
+      contents = spki_pkcs5_decrypt(mac, crypto,
+				    options->super.tty,
+				    contents);
+      if (!contents)
         {
           werror("Decrypting private key failed.\n");
           return make_object_list(0, -1);
         }
     }
   
-  s = spki_sexp_to_signer(options->signature_algorithms, expr,
-                          &algorithm_name);
+  s = spki_make_signer(options->signature_algorithms, contents,
+		       &algorithm_name);
+
+  lsh_string_free(contents);
+  
   if (!s)
     {
       werror("Invalid private key.\n");
@@ -331,8 +359,7 @@ read_user_keys(struct lsh_options *options)
   v = SIGNER_GET_VERIFIER(s);
   assert(v);
   
-  spki_public = sexp_format(spki_make_public_key(v),
-			    SEXP_CANONICAL, 0);
+  spki_public = PUBLIC_SPKI_KEY(v, 0);
 
   /* Test key here? */  
   switch (algorithm_name)
@@ -395,7 +422,7 @@ STATIC_COMMAND(do_options2identities);
      (vars
        (db object spki_context)
        (tty object interact)
-       (access object sexp)
+       (access string)
        (host object address_info)
        ; Allow unauthorized keys
        (sloppy . int)
@@ -412,38 +439,49 @@ do_lsh_lookup(struct lookup_verifier *c,
 	      struct lsh_string *key)
 {
   CAST(lsh_host_db, self, c);
-  struct spki_subject *subject;
+  struct spki_principal *subject;
 
   switch (method)
     {
     case ATOM_SSH_DSS:
-      {
+      {	
+	struct lsh_string *spki_key;
 	struct verifier *v = make_ssh_dss_verifier(key->length, key->data);
+
 	if (!v)
 	  {
 	    werror("do_lsh_lookup: Invalid ssh-dss key.\n");
 	    return NULL;
 	  }
-	subject = SPKI_LOOKUP(self->db,
-			      spki_make_public_key(v),
-			      v);
+
+	/* FIXME: It seems like a waste to pick apart the sexp again */
+	spki_key = PUBLIC_SPKI_KEY(v, 0);
+
+	subject = spki_lookup(self->db, spki_key->length, spki_key->data, v);
 	assert(subject);
 	assert(subject->verifier);
+
+	lsh_string_free(spki_key);
 	break;
       }
     case ATOM_SSH_RSA:
       {
+	struct lsh_string *spki_key;
 	struct verifier *v = make_ssh_rsa_verifier(key->length, key->data);
+
 	if (!v)
 	  {
 	    werror("do_lsh_lookup: Invalid ssh-rsa key.\n");
 	    return NULL;
 	  }
-	subject = SPKI_LOOKUP(self->db,
-			      spki_make_public_key(v),
-			      v);
+
+	/* FIXME: It seems like a waste to pick apart the sexp again */
+	spki_key = PUBLIC_SPKI_KEY(v, 0);
+	subject = spki_lookup(self->db, spki_key->length, spki_key->data, v);
 	assert(subject);
 	assert(subject->verifier);
+
+	lsh_string_free(spki_key);
 	break;
       }
       
@@ -470,14 +508,7 @@ do_lsh_lookup(struct lookup_verifier *c,
     case ATOM_SPKI_SIGN_RSA:
     case ATOM_SPKI_SIGN_DSS:
       {
-	struct sexp *e = string_to_sexp(SEXP_CANONICAL, key, 0);
-	if (!e)
-	  {
-	    werror("do_lsh_lookup: Invalid spki s-expression.\n");
-	    return NULL;
-	  }
-	  
-	subject = SPKI_LOOKUP(self->db, e, NULL);
+	subject = spki_lookup(self->db, key->length, key->data, NULL);
 	if (!subject)
 	  {
 	    werror("do_lsh_lookup: Invalid spki key.\n");
@@ -499,13 +530,14 @@ do_lsh_lookup(struct lookup_verifier *c,
   
   /* Check authorization */
 
-  if (SPKI_AUTHORIZE(self->db, subject, self->access))
+  if (spki_authorize(self->db, subject, time(NULL), self->access))
     {
       verbose("SPKI host authorization successful!\n");
     }
   else
     {
-      struct sexp *acl;
+      struct lsh_string *acl;
+      struct spki_iterator i;
       
       verbose("SPKI authorization failed.\n");
       if (!self->sloppy)
@@ -518,13 +550,12 @@ do_lsh_lookup(struct lookup_verifier *c,
       if (!quiet_flag)
 	{
 	  /* Display fingerprint */
-
+	  /* FIXME: Rewrite to use libspki subject */
+#if 0
 	  struct lsh_string *spki_fingerprint = 
-	    hash_string(self->hash,
-			sexp_format(subject->key, SEXP_CANONICAL, 0),
-			1);
-
-
+	    hash_string(self->hash, subject->key, 0);
+#endif
+	  
 	  struct lsh_string *fingerprint = 
 	    lsh_string_colonize( 
 				ssh_format( "%lfxS", 
@@ -550,30 +581,31 @@ do_lsh_lookup(struct lookup_verifier *c,
 			  "Key details:\n"
 			  "Bubble Babble: %lfS\n"
 			  "Fingerprint:   %lfS\n"
-			  "SPKI SHA1:     %lfxS\n"
+			  /* "SPKI SHA1:     %lfxS\n" */
 			  "Do you trust this key? (y/n) ",
-			  self->host->ip, babble, fingerprint, spki_fingerprint), 0, 1))
+			  self->host->ip, babble, fingerprint /* , spki_fingerprint */), 0, 1))
 	    return NULL;
 	}
 
-      acl = sexp_l(2, sexp_a(ATOM_ACL),
-		   sexp_l(3, sexp_a(ATOM_ENTRY),
-			  subject->key,
-			  sexp_l(2, sexp_a(ATOM_TAG),
-				 self->access,
-				 -1),
-			  -1),
-		   -1);
-
-      /* Remember this key. We don't want to ask again for key re-exchange */
-      spki_add_acl(self->db, acl);
+      acl = lsh_sexp_format(0, "(acl(entry(subject%l%l)))",
+			    subject->key_length, subject->key,
+			    self->access->length, self->access->data);
       
+      /* FIXME: Seems awkward to pick the acl apart again. */
+      if (!spki_iterator_first(&i, acl->length, acl->data))
+	fatal("Internal error.\n");
+      
+      /* Remember this key. We don't want to ask again for key re-exchange */
+      spki_add_acl(self->db, &i);
+
       /* Write an ACL to disk. */
       if (self->file)
 	{
 	  A_WRITE(self->file, ssh_format("\n; ACL for host %lS\n", self->host->ip));
 	  A_WRITE(self->file,
-		  sexp_format(acl, SEXP_TRANSPORT, 0));
+		  lsh_sexp_format(1, "%l", acl->length, acl->data));
+	  lsh_string_free(acl);
+	  
 	  A_WRITE(self->file, ssh_format("\n"));
 	}
     }
@@ -735,7 +767,7 @@ main_options[] =
     "Try publickey user authentication (default).", 0 },
   { "no-publickey", OPT_PUBLICKEY | ARG_NOT, NULL, 0,
     "Don't try publickey user authentication.", 0 },
-  { "host-db", OPT_HOST_DB, "Filename", 0, "By default, ~/.lsh/known_hosts", 0},
+  { "host-db", OPT_HOST_DB, "Filename", 0, "By default, ~/.lsh/host-acls", 0},
   { "sloppy-host-authentication", OPT_SLOPPY, NULL, 0,
     "Allow untrusted hostkeys.", 0 },
   { "strict-host-authentication", OPT_STRICT, NULL, 0,
@@ -825,6 +857,9 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       break;
       
     case ARGP_KEY_END:
+      if (self->super.inhibit_actions)
+	break;
+
       if (!self->home)
 	{
 	  argp_error(state, "No home directory. Please set HOME in the environment.");
@@ -969,7 +1004,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 
     case 'R':
       {
-	UINT32 listen_port;
+	uint32_t listen_port;
 	struct address_info *target;
 
 	if (!client_parse_forward_arg(arg, &listen_port, &target))
@@ -1035,8 +1070,6 @@ do_lsh_default_handler(struct exception_handler *s,
       {
       case EXC_RESOLVE:
       case EXC_USERAUTH:
-      case EXC_SEXP_SYNTAX:
-      case EXC_SPKI_TYPE:
       case EXC_GLOBAL_REQUEST:
       case EXC_CHANNEL_REQUEST:
       case EXC_CHANNEL_OPEN:
@@ -1065,7 +1098,7 @@ make_lsh_default_handler(int *status, struct exception_handler *parent,
 }
 
 
-int main(int argc, char **argv)
+int main(int argc, char **argv, const char** envp)
 {
   struct lsh_options *options;
   struct spki_context *spki;
@@ -1091,6 +1124,7 @@ int main(int argc, char **argv)
   if (!options)
     return EXIT_FAILURE;
   
+  envp_parse(&main_argp, envp, "LSHFLAGS=", ARGP_IN_ORDER, options);
   argp_parse(&main_argp, argc, argv, ARGP_IN_ORDER, NULL, options);
 
   if (!options->super.random)
