@@ -1,8 +1,8 @@
 /* lshd.c
  *
- * main server program.
+ * Main server program.
  *
- * $Id: lshd.c,v 1.144 2002/08/26 20:14:18 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -33,7 +33,7 @@
 #include "connection_commands.h"
 #include "crypto.h"
 #include "daemon.h"
-#include "dsa.h"
+#include "environ.h"
 #include "format.h"
 #include "handshake.h"
 #include "io.h"
@@ -46,7 +46,6 @@
 #include "server_keyexchange.h"
 #include "server_pty.h"
 #include "server_session.h"
-#include "sexp.h"
 #include "spki.h"
 #include "srp.h"
 #include "ssh.h"
@@ -85,6 +84,12 @@ struct command_2 close_on_sighup;
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#if TIME_WITH_SYS_TIME && HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#if HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -97,13 +102,6 @@ const char *argp_program_version
 
 const char *argp_program_bug_address = BUG_ADDRESS;
 
-/* The definition of SBINDIR is currently broken */
-#if 0
-# define KERBEROS_HELPER SBINDIR "/lsh-krb-checkpw"
-#else
-# define KERBEROS_HELPER PREFIX "/sbin/lsh-krb-checkpw"
-#endif
-
 #define OPT_NO 0x400
 #define OPT_SSH1_FALLBACK 0x200
 #define OPT_INTERFACE 0x201
@@ -161,7 +159,6 @@ const char *argp_program_bug_address = BUG_ADDRESS;
        (random object randomness)
        
        (signature_algorithms object alist)
-       (style . sexp_argp_state)
        (interface . "char *")
        (port . "char *")
        (hostkey . "char *")
@@ -208,8 +205,6 @@ do_exc_lshd_handler(struct exception_handler *s,
   switch(e->type)
     {
     case EXC_RESOLVE:
-    case EXC_SEXP_SYNTAX:
-    case EXC_SPKI_TYPE:
     case EXC_RANDOMNESS_LOW_ENTROPY:
       werror("lshd: %z\n", e->msg);
       exit(EXIT_FAILURE);
@@ -239,7 +234,6 @@ make_lshd_options(void)
 
   self->signature_algorithms = all_signature_algorithms(self->random); /* OK to initialize with NULL */
 
-  self->style = SEXP_TRANSPORT;
   self->interface = NULL;
 
   /* Default behaviour is to lookup the "ssh" service, and fall back
@@ -486,7 +480,7 @@ main_options[] =
   
   { "kerberos-passwords", OPT_KERBEROS_PASSWD, NULL, 0,
     "Recognize kerberos passwords, using the helper program "
-    "\"" KERBEROS_HELPER "\". This option is experimental.", 0 },
+    "\"" PATH_KERBEROS_HELPER "\". This option is experimental.", 0 },
   { "no-kerberos-passwords", OPT_NO_KERBEROS_PASSWD, NULL, 0,
     "Don't recognize kerberos passwords (default behaviour).", 0 },
 
@@ -532,7 +526,6 @@ main_options[] =
 static const struct argp_child
 main_argp_children[] =
 {
-  { &sexp_input_argp, 0, "", 0 },
   { &algorithms_argp, 0, "", 0 },
   { &werror_argp, 0, "", 0 },
   { NULL, 0, NULL, 0}
@@ -597,9 +590,8 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
     default:
       return ARGP_ERR_UNKNOWN;
     case ARGP_KEY_INIT:
-      state->child_inputs[0] = &self->style;
-      state->child_inputs[1] = &self->super;
-      state->child_inputs[2] = NULL;
+      state->child_inputs[0] = &self->super;
+      state->child_inputs[1] = NULL;
       break;
     case ARGP_KEY_END:
       {
@@ -755,7 +747,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       break;
 
     case OPT_KERBEROS_PASSWD:
-      self->pw_helper = KERBEROS_HELPER;
+      self->pw_helper = PATH_KERBEROS_HELPER;
       break;
 
     case OPT_NO_KERBEROS_PASSWD:
@@ -925,6 +917,17 @@ main(int argc, char **argv)
 {
   struct lshd_options *options;
 
+#if HAVE_SETRLIMIT && HAVE_SYS_RESOURCE_H
+  /* Try to increase max number of open files, ignore any error */
+
+  struct rlimit r;
+
+  r.rlim_max = RLIM_INFINITY;
+  r.rlim_cur = RLIM_INFINITY;
+
+  setrlimit(RLIMIT_NOFILE, &r);
+#endif
+
   io_init();
   
   /* For filtering messages. Could perhaps also be used when converting
@@ -934,6 +937,7 @@ main(int argc, char **argv)
   /* FIXME: Choose character set depending on the locale */
   set_local_charset(CHARSET_LATIN1);
 
+
   options = make_lshd_options();
 
   if (!options)
