@@ -1,15 +1,16 @@
 /* memxor.c
  *
- * $Id: memxor.c,v 1.1 1999/02/17 22:37:50 nisse Exp $
  */
 
 /* XOR LEN bytes starting at SRCADDR onto DESTADDR.  Result undefined
    if the source overlaps with the destination.
    Return DESTADDR. */
 
+#include <inttypes.h>
+
 #include "memxor.h"
 
-UINT8 *memxor(UINT8 *dst, const UINT8 *src, size_t n)
+uint8_t *memxor(uint8_t *dst, const uint8_t *src, size_t n)
 {
   size_t i;
   for (i = 0; i<n; i++)
