@@ -5,7 +5,7 @@
 
 /* nettle, low-level cryptographics library
  *
- * Copyright (C) 2001 Niels Mller
+ * Copyright (C) 2001, 2002 Niels Mller
  *  
  * The nettle library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -32,11 +32,8 @@
 #include "md5.h"
 #include "sha.h"
 
-/* Randomness function. This typedef doesn't really belong here, but
- * so far it's used only by rsa functions (encryption and key
- * generation) */
-typedef void (*nettle_random_func)(void *ctx,
-				   unsigned length, uint8_t *dst);
+/* For nettle_random_func */
+#include "nettle-meta.h"
 
 
 /* For PKCS#1 to make sense, the size of the modulo, in octets, must
@@ -87,7 +84,7 @@ struct rsa_private_key
  *
  * Store the private key in a rsa_private_key struct.
  *
- * Call rsa_prepare_private_key. This initializes the size attribute
+ * Call rsa_private_key_prepare. This initializes the size attribute
  * to the length of a signature.
  *
  * Initialize a hashing context, by callling
@@ -105,52 +102,73 @@ struct rsa_private_key
  * When done with the key and signature, don't forget to call
  * mpz_clear.
  */
-
+ 
 /* Calls mpz_init to initialize bignum storage. */
 void
-rsa_init_public_key(struct rsa_public_key *key);
+rsa_public_key_init(struct rsa_public_key *key);
 
 /* Calls mpz_clear to deallocate bignum storage. */
 void
-rsa_clear_public_key(struct rsa_public_key *key);
+rsa_public_key_clear(struct rsa_public_key *key);
 
 int
-rsa_prepare_public_key(struct rsa_public_key *key);
+rsa_public_key_prepare(struct rsa_public_key *key);
 
 /* Calls mpz_init to initialize bignum storage. */
 void
-rsa_init_private_key(struct rsa_private_key *key);
+rsa_private_key_init(struct rsa_private_key *key);
 
 /* Calls mpz_clear to deallocate bignum storage. */
 void
-rsa_clear_private_key(struct rsa_private_key *key);
+rsa_private_key_clear(struct rsa_private_key *key);
 
 int
-rsa_prepare_private_key(struct rsa_private_key *key);
+rsa_private_key_prepare(struct rsa_private_key *key);
 
 
 /* PKCS#1 style signatures */
 void
-rsa_md5_sign(struct rsa_private_key *key,
+rsa_md5_sign(const struct rsa_private_key *key,
              struct md5_ctx *hash,
              mpz_t signature);
 
 
 int
-rsa_md5_verify(struct rsa_public_key *key,
+rsa_md5_verify(const struct rsa_public_key *key,
                struct md5_ctx *hash,
 	       const mpz_t signature);
 
 void
-rsa_sha1_sign(struct rsa_private_key *key,
+rsa_sha1_sign(const struct rsa_private_key *key,
               struct sha1_ctx *hash,
               mpz_t signature);
 
 int
-rsa_sha1_verify(struct rsa_public_key *key,
+rsa_sha1_verify(const struct rsa_public_key *key,
                 struct sha1_ctx *hash,
 		const mpz_t signature);
 
+/* Variants taking the digest as argument. */
+void
+rsa_md5_sign_digest(const struct rsa_private_key *key,
+		    const uint8_t *digest,
+		    mpz_t s);
+
+int
+rsa_md5_verify_digest(const struct rsa_public_key *key,
+		      const uint8_t *digest,
+		      const mpz_t signature);
+
+void
+rsa_sha1_sign_digest(const struct rsa_private_key *key,
+		     const uint8_t *digest,
+		     mpz_t s);
+
+int
+rsa_sha1_verify_digest(const struct rsa_public_key *key,
+		       const uint8_t *digest,
+		       const mpz_t signature);
+
 
 /* RSA encryption, using PKCS#1 */
 /* FIXME: These functions uses the v1.5 padding. What should the v2
@@ -159,7 +177,7 @@ rsa_sha1_verify(struct rsa_public_key *key,
 /* Returns 1 on success, 0 on failure, which happens if the
  * message is too long for the key. */
 int
-rsa_encrypt(struct rsa_public_key *key,
+rsa_encrypt(const struct rsa_public_key *key,
 	    /* For padding */
 	    void *random_ctx, nettle_random_func random,
 	    unsigned length, const uint8_t *cleartext,
@@ -171,22 +189,18 @@ rsa_encrypt(struct rsa_public_key *key,
  * failure, which happens if decryption failed or if the message
  * didn't fit. */
 int
-rsa_decrypt(struct rsa_private_key *key,
+rsa_decrypt(const struct rsa_private_key *key,
 	    unsigned *length, uint8_t *cleartext,
 	    const mpz_t ciphertext);
 
-
 /* Compute x, the e:th root of m. Calling it with x == m is allowed. */
 void
-rsa_compute_root(struct rsa_private_key *key, mpz_t x, const mpz_t m);
+rsa_compute_root(const struct rsa_private_key *key,
+		 mpz_t x, const mpz_t m);
 
 
 /* Key generation */
 
-/* Progress report function. */
-typedef void (*nettle_progress_func)(void *ctx,
-				     int c);
-
 /* Note that the key structs must be initialized first. */
 int
 rsa_generate_keypair(struct rsa_public_key *pub,
@@ -213,4 +227,52 @@ rsa_generate_keypair(struct rsa_public_key *pub,
   rsa_##algorithm##_verify(key, ctx, signature) \
 )
 
+
+/* Keys in sexp form. */
+
+struct nettle_buffer;
+
+/* Generates a public-key expression if PRIV is NULL .*/
+int
+rsa_keypair_to_sexp(struct nettle_buffer *buffer,
+		    const char *algorithm_name, /* NULL means "rsa" */
+		    const struct rsa_public_key *pub,
+		    const struct rsa_private_key *priv);
+
+struct sexp_iterator;
+
+int
+rsa_keypair_from_sexp_alist(struct rsa_public_key *pub,
+			    struct rsa_private_key *priv,
+			    unsigned limit,
+			    struct sexp_iterator *i);
+
+/* If PRIV is NULL, expect a public-key expression. If PUB is NULL,
+ * expect a private key expression and ignore the parts not needed for
+ * the public key. */
+/* Keys must be initialized before calling this function, as usual. */
+int
+rsa_keypair_from_sexp(struct rsa_public_key *pub,
+		      struct rsa_private_key *priv,
+		      unsigned limit,
+		      unsigned length, const uint8_t *expr);
+
+
+/* OpenPGP format. Experimental interface, subject to change. */
+int
+rsa_keypair_to_openpgp(struct nettle_buffer *buffer,
+		       const struct rsa_public_key *pub,
+		       const struct rsa_private_key *priv,
+		       /* A single user id. NUL-terminated utf8. */
+		       const char userid);
+
+/* Internal functions. */
+int
+_rsa_verify(const struct rsa_public_key *key,
+	    const mpz_t m,
+	    const mpz_t s);
+
+unsigned
+_rsa_check_size(mpz_t n);
+
 #endif /* NETTLE_RSA_H_INCLUDED */
