@@ -4,7 +4,11 @@
 
 #include "crypto.h"
 
-#include <getopt.h>
+/* FIXME: In which include file can getopt be found? Solaris man page
+ * says stdlib.h, linux's says unistd.h. */
+#include <stdlib.h>
+#include <unistd.h>
+
 #include <stdio.h>
 #include <string.h>
 
@@ -72,7 +76,7 @@ int main(int argc, char **argv)
   
   key = alloca(length);
   
-  pkcs5_derive_key(make_hmac_algorithm(&sha1_algorithm),
+  pkcs5_derive_key(make_hmac_algorithm(&crypto_sha1_algorithm),
 		   strlen(password), password,
 		   strlen(salt), salt,
 		   iterations,
