@@ -1,6 +1,6 @@
 /* proxy_session.h
  *
- * $Id: proxy_session.h,v 1.3 2000/01/22 21:34:05 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -31,7 +31,4 @@ struct channel_open *
 make_proxy_open_session(struct alist *server_requests,
 			struct alist *client_requests);
 
-struct channel_request *
-make_proxy_channel_request(void);
-
 #endif /* LSH_PROXY_SESSION_H_INCLUDED */
