@@ -1,6 +1,4 @@
 /* queue.h
- *
- * $Id: queue.h,v 1.11 2001/03/12 21:37:13 nisse Exp $
  *
  * Generic doubly linked list. */
 
@@ -84,16 +82,11 @@ struct object_queue_node
    (struct
      (name object_queue)
      (vars
-       (length . UINT32)
+       (length . uint32_t)
        (q indirect-special "struct lsh_queue"
           do_object_queue_mark do_object_queue_free)))
 */
 
-#if 0
-#define object_queue_is_empty(x) (lsh_queue_is_empty(&(x)->q))
-#define object_queue_init(x) (lsh_queue_init(&(x)->q))
-#endif
-
 void object_queue_init(struct object_queue *q);
 int object_queue_is_empty(struct object_queue *q);
 
@@ -140,7 +133,7 @@ struct string_queue_node
    (struct
      (name string_queue)
      (vars
-       (length . UINT32)
+       (length . uint32_t)
        (q indirect-special "struct lsh_queue"
           #f do_string_queue_free)))
 */
