@@ -1,6 +1,5 @@
 /* rsa.c
  *
- * $Id: rsa.c,v 1.20 2002/06/04 19:23:28 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -22,6 +21,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <string.h>
+
 #include "publickey_crypto.h"
 
 #include "atoms.h"
@@ -31,21 +37,19 @@
 #include "werror.h"
 #include "xalloc.h"
 
+#include "nettle/bignum.h"
 #include "nettle/rsa.h"
+#include "nettle/sexp.h"
 #include "nettle/sha.h"
 
-#include <assert.h>
-#include <string.h>
-
 #include "rsa.c.x"
 
-#define SA(x) sexp_a(ATOM_##x)
-
 /* We don't allow keys larger than 5000 bits (i.e. 625 octets). Note
  * that allowing really large keys opens for Denial-of-service
  * attacks. */
 
-#define RSA_MAX_SIZE 625
+#define RSA_MAX_OCTETS 625
+#define RSA_MAX_BITS (8 * RSA_MAX_OCTETS)
 
 /* GABA:
    (class
@@ -53,7 +57,7 @@
      (super verifier)
      (vars
        (key indirect-special "struct rsa_public_key"
-            #f rsa_clear_public_key)))
+            #f rsa_public_key_clear)))
 */
 
 /* GABA:
@@ -63,41 +67,17 @@
      (vars
        (verifier object rsa_verifier)
        (key indirect-special "struct rsa_private_key"
-            #f rsa_clear_private_key)))
+            #f rsa_private_key_clear)))
 */
 
-
-/* FIXME: Add hash algorithm to signature value? */
-static struct sexp *
-encode_rsa_sig_val(mpz_t s)
-{
-  return sexp_un(s);
-}
-
-static int
-decode_rsa_sig_val(struct sexp *e, mpz_t s, unsigned limit)
-{
-  return sexp2bignum_u(e, s, limit);
-}
-
-static int
-spki_init_rsa_verifier(struct rsa_public_key *key,
-		       struct sexp_iterator *i)
-{
-  return (sexp_get_un(i, ATOM_N, key->n, RSA_MAX_SIZE)
-	  && sexp_get_un(i, ATOM_E, key->e, RSA_MAX_SIZE)
-	  && rsa_prepare_public_key(key));
-}
-
-
 /* NOTE: For now, always use sha1. */
 static int
 do_rsa_verify(struct verifier *v,
 	      int algorithm,
-	      UINT32 length,
-	      const UINT8 *msg,
-	      UINT32 signature_length,
-	      const UINT8 *signature_data)
+	      uint32_t length,
+	      const uint8_t *msg,
+	      uint32_t signature_length,
+	      const uint8_t *signature_data)
 {
   CAST(rsa_verifier, self, v);
   struct sha1_ctx hash;
@@ -114,8 +94,8 @@ do_rsa_verify(struct verifier *v,
     case ATOM_SSH_RSA:
       {
 	struct simple_buffer buffer;
-	UINT32 length;
-	const UINT8 *digits;
+	uint32_t length;
+	const uint8_t *digits;
 	int atom;
 	
 	simple_buffer_init(&buffer, signature_length, signature_data);
@@ -127,7 +107,7 @@ do_rsa_verify(struct verifier *v,
 	      && parse_eod(&buffer) ))
 	  goto fail;
 
-	bignum_parse_u(s, length, digits);
+	nettle_mpz_set_str_256_u(s, length, digits);
 
 	break;
       }
@@ -135,15 +115,12 @@ do_rsa_verify(struct verifier *v,
       /* It doesn't matter here which flavour of SPKI is used. */
     case ATOM_SPKI_SIGN_RSA:
     case ATOM_SPKI_SIGN_DSS:
+    case ATOM_SPKI:
       {
-	struct simple_buffer buffer;
-	struct sexp *e;
-	
-	simple_buffer_init(&buffer, signature_length, signature_data);
+	struct sexp_iterator i;
 
-	if (! ( (e = sexp_parse_canonical(&buffer))
-		&& parse_eod(&buffer)
-		&& decode_rsa_sig_val(e, s, self->key.size)) )
+	if (! (sexp_iterator_first(&i, signature_length, signature_data)
+	       && nettle_mpz_set_sexp(s, 8 * self->key.size, &i)))
 	  goto fail;
 
 	break;
@@ -163,32 +140,6 @@ do_rsa_verify(struct verifier *v,
   return res;
 }
 
-/* FIXME: Add hash algorithm to signature value? */
-static int
-do_rsa_verify_spki(struct verifier *v,
-		   UINT32 length,
-		   const UINT8 *msg,
-		   struct sexp *e)
-{
-  CAST(rsa_verifier, self, v);
-  struct sha1_ctx hash;
-  
-  mpz_t s;
-  int res;
-  
-  mpz_init(s);
-
-  sha1_init(&hash);
-  sha1_update(&hash, length, msg);
-  
-  res = (decode_rsa_sig_val(e, s, self->key.size)
-	 && rsa_sha1_verify(&self->key, &hash, s));
-  
-  mpz_clear(s);
-
-  return res;
-}
-
 static struct lsh_string *
 do_rsa_public_key(struct verifier *s)
 {
@@ -198,16 +149,11 @@ do_rsa_public_key(struct verifier *s)
 		     self->key.e, self->key.n);
 }
 
-static struct sexp *
-do_rsa_public_spki_key(struct verifier *s)
+static struct lsh_string *
+do_rsa_public_spki_key(struct verifier *s, int transport)
 {
   CAST(rsa_verifier, self, s);
-#if 0
-  return sexp_l(3, sexp_a(self->params->name),
-		sexp_l(2, SA(N), sexp_un(self->n), -1),
-		sexp_l(2, SA(E), sexp_un(self->e), -1),
-		-1);
-#endif
+
   /* NOTE: The algorithm name "rsa-pkcs1-sha1" is the SPKI standard,
    * and what lsh-1.2 used. "rsa-pkcs1" makes more sense, and is what
    * gnupg uses internally (I think), and was used by some late
@@ -215,10 +161,9 @@ do_rsa_public_spki_key(struct verifier *s)
    *
    * However, since it doesn't matter much, for now we follow the SPKI
    * standard and stay compatible with lsh-1.2. */
-  return sexp_l(3, sexp_a(ATOM_RSA_PKCS1_SHA1),
-		sexp_l(2, SA(N), sexp_un(self->key.n), -1),
-		sexp_l(2, SA(E), sexp_un(self->key.e), -1),
-		-1);
+  /* FIXME: Use nettle's rsa_keypair_to_sexp. */
+  return lsh_sexp_format(transport, "(public-key(rsa-pkcs1-sha1(n%b)(e%b)))",
+			 self->key.n, self->key.e);
 }
 
 
@@ -233,33 +178,13 @@ init_rsa_verifier(struct rsa_verifier *self)
 {
   /* FIXME: The allocator could do this kind of initialization
    * automatically. */
-  rsa_init_public_key(&self->key);
+  rsa_public_key_init(&self->key);
   
   self->super.verify = do_rsa_verify;
-  self->super.verify_spki = do_rsa_verify_spki;
   self->super.public_key = do_rsa_public_key;
   self->super.public_spki_key = do_rsa_public_spki_key;
 }
 
-static struct rsa_verifier *
-make_rsa_verifier_internal(struct sexp_iterator *i)
-{
-  NEW(rsa_verifier, res);
-  init_rsa_verifier(res);
-
-  assert(SEXP_LEFT(i) >= 2);
-  
-  if (spki_init_rsa_verifier(&res->key, i))
-    {
-      return res;
-    }
-  else
-    {
-      KILL(res);
-      return NULL;
-    }
-}
-  
 /* Alternative constructor using a key of type ssh-rsa, when the atom
  * "ssh-rsa" is already read from the buffer. */
 struct verifier *
@@ -268,13 +193,13 @@ parse_ssh_rsa_public(struct simple_buffer *buffer)
   NEW(rsa_verifier, res);
   init_rsa_verifier(res);
 
-  if (parse_bignum(buffer, res->key.e, RSA_MAX_SIZE)
+  if (parse_bignum(buffer, res->key.e, RSA_MAX_OCTETS)
       && (mpz_sgn(res->key.e) == 1)
-      && parse_bignum(buffer, res->key.n, RSA_MAX_SIZE)
+      && parse_bignum(buffer, res->key.n, RSA_MAX_OCTETS)
       && (mpz_sgn(res->key.n) == 1)
       && (mpz_cmp(res->key.e, res->key.n) < 0)
       && parse_eod(buffer)
-      && rsa_prepare_public_key(&res->key))
+      && rsa_public_key_prepare(&res->key))
     return &res->super;
 
   else
@@ -289,8 +214,8 @@ parse_ssh_rsa_public(struct simple_buffer *buffer)
 static struct lsh_string *
 do_rsa_sign(struct signer *s,
 	    int algorithm,
-	    UINT32 msg_length,
-	    const UINT8 *msg)
+	    uint32_t msg_length,
+	    const uint8_t *msg)
 {
   CAST(rsa_signer, self, s);
   struct lsh_string *res;
@@ -320,8 +245,9 @@ do_rsa_sign(struct signer *s,
       /* It doesn't matter here which flavour of SPKI is used. */
     case ATOM_SPKI_SIGN_RSA:
     case ATOM_SPKI_SIGN_DSS:
-
-      res = sexp_format(encode_rsa_sig_val(signature), SEXP_CANONICAL, 0);
+    case ATOM_SPKI:
+      /* FIXME: Add hash algorithm to signature value? */
+      res = lsh_sexp_format(0, "%b", signature);
       break;
     default:
       fatal("do_rsa_sign: Internal error!\n");
@@ -330,31 +256,6 @@ do_rsa_sign(struct signer *s,
   return res;
 }
 
-static struct sexp *
-do_rsa_sign_spki(struct signer *s,
-		 /* struct sexp *hash, struct sexp *principal, */
-		 UINT32 msg_length,
-		 const UINT8 *msg)
-{
-  CAST(rsa_signer, self, s);
-  struct sha1_ctx hash;
-  
-  mpz_t x;
-  struct sexp *signature;
-  
-  mpz_init(x);
-
-  sha1_init(&hash);
-  sha1_update(&hash, msg_length, msg);
-
-  rsa_sha1_sign(&self->key, &hash, x);
-  
-  signature = encode_rsa_sig_val(x);
-  
-  mpz_clear(x);
-  return signature;
-}
-
 static struct verifier *
 do_rsa_get_verifier(struct signer *s)
 {
@@ -367,9 +268,15 @@ static struct verifier *
 make_rsa_verifier(struct signature_algorithm *s UNUSED,
 		  struct sexp_iterator *i)
 {
-  return ( (SEXP_LEFT(i) == 2)
-	   ? &make_rsa_verifier_internal(i)->super
-	   : NULL);
+  NEW(rsa_verifier, res);
+  init_rsa_verifier(res);
+
+  if (rsa_keypair_from_sexp_alist(&res->key, NULL, RSA_MAX_BITS, i)
+      && rsa_public_key_prepare(&res->key))
+    return &res->super;
+
+  KILL(res);
+  return NULL;
 }
 
 
@@ -377,48 +284,35 @@ static struct signer *
 make_rsa_signer(struct signature_algorithm *s UNUSED,
 		struct sexp_iterator *i)
 {
+  NEW(rsa_verifier, verifier);
   NEW(rsa_signer, res);
 
-  rsa_init_private_key(&res->key);
-
-  /* The secret d is optional, it's not needed. */
-  if ( (SEXP_LEFT(i) >= 7)
-       && ( (res->verifier = make_rsa_verifier_internal(i)) )
-       && sexp_get_un(i, ATOM_P, res->key.p, RSA_MAX_SIZE)
-       && sexp_get_un(i, ATOM_Q, res->key.q, RSA_MAX_SIZE)
-       && sexp_get_un(i, ATOM_A, res->key.a, RSA_MAX_SIZE)
-       && sexp_get_un(i, ATOM_B, res->key.b, RSA_MAX_SIZE)
-       && sexp_get_un(i, ATOM_C, res->key.c, RSA_MAX_SIZE) )
+  init_rsa_verifier(verifier);
+  
+  rsa_private_key_init(&res->key);
+  if (rsa_keypair_from_sexp_alist(&verifier->key, &res->key, RSA_MAX_BITS, i)
+      && rsa_public_key_prepare(&verifier->key)
+      && rsa_private_key_prepare(&res->key)
+      && res->key.size == verifier->key.size)
     {
-      if (!rsa_prepare_private_key(&res->key)
-	  || (res->key.size != res->verifier->key.size))
-	{
-	  KILL(res->verifier);
-	  res->verifier = NULL;
-	  KILL(res);
-
-	  return NULL;
-	}
+      res->verifier = verifier;
 
       res->super.sign = do_rsa_sign;
-      res->super.sign_spki = do_rsa_sign_spki;
       res->super.get_verifier = do_rsa_get_verifier;
       
       return &res->super;
     }
-  else
-    {
-      KILL(res);
-      return NULL;
-    }
+  KILL(res);
+  KILL(res->verifier);
+  return NULL;
 }
 
 struct signature_algorithm rsa_sha1_algorithm =
   { STATIC_HEADER, make_rsa_signer, make_rsa_verifier };
 
 struct verifier *
-make_ssh_rsa_verifier(UINT32 public_length,
-		      const UINT8 *public)
+make_ssh_rsa_verifier(uint32_t public_length,
+		      const uint8_t *public)
 {
   struct simple_buffer buffer;
   int atom;
