@@ -1,6 +1,6 @@
 /* server_keyexchange.c
  *
- * $Id: server_keyexchange.c,v 1.51 2001/10/02 22:43:33 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -26,7 +26,6 @@
 #include "atoms.h"
 #include "command.h"
 #include "debug.h"
-#include "dsa.h"
 #include "format.h"
 #include "sexp.h"
 #include "srp.h"
@@ -78,11 +77,11 @@ do_handle_dh_init(struct packet_handler *c,
     }
   
   /* Send server's message, to complete key exchange */
-  C_WRITE(connection,
-	  dh_make_server_msg(&closure->dh,
-			     closure->server_key,
-			     closure->hostkey_algorithm,
-			     closure->signer));
+  C_WRITE_NOW(connection,
+	      dh_make_server_msg(&closure->dh,
+				 closure->server_key,
+				 closure->hostkey_algorithm,
+				 closure->signer));
 
   connection->dispatch[SSH_MSG_KEXDH_INIT] = &connection_fail_handler;
 
@@ -195,7 +194,7 @@ do_srp_server_proof_handler(struct packet_handler *s,
   
   if (response)
     {
-      C_WRITE(connection, response);
+      C_WRITE_NOW(connection, response);
 
       /* Remember that a user was authenticated. */
       connection->user = self->srp->user;
@@ -261,26 +260,19 @@ do_server_srp_read_verifier(struct abstract_write *s,
   
   else
     {
-      struct sexp *expr = string_to_sexp(SEXP_CANONICAL, file, 1);
-
-      if (!expr)
-	disconnect_kex_failed(self->connection,
-				"S-expression Syntax error in verifier file");
+      self->srp->entry = make_srp_entry(self->srp->user->name, file);
+	  
+      if (!self->srp->entry)
+	disconnect_kex_failed(self->connection, "Invalid SRP verifier");
       else
 	{
-	  self->srp->entry = make_srp_entry(self->srp->user->name, expr);
-	  
-	  if (!self->srp->entry)
-	    disconnect_kex_failed(self->connection, "Invalid SRP verifier");
-	  else
-	    {
-	      /* Success */
+	  /* Success */
 
-	      C_WRITE(self->connection,
-		      srp_make_reply_msg(&self->srp->dh, self->srp->entry));
-	      self->connection->dispatch[SSH_MSG_KEXSRP_PROOF]
-		= make_srp_server_proof_handler(self->srp);
-	    }
+	  C_WRITE_NOW(self->connection,
+		      srp_make_reply_msg(&self->srp->dh,
+					 self->srp->entry));
+	  self->connection->dispatch[SSH_MSG_KEXSRP_PROOF]
+	    = make_srp_server_proof_handler(self->srp);
 	}
     }
 }
