@@ -1,6 +1,5 @@
 /* server_userauth.h
  *
- * $Id: server_userauth.h,v 1.19 2001/09/24 16:51:13 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -25,6 +24,11 @@
 #ifndef LSH_SERVER_USERAUTH_H_INCLUDED
 #define LSH_SERVER_USERAUTH_H_INCLUDED
 
+/* For uid_t and gid_t */
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "alist.h"
 #include "command.h"
 #include "connection.h"
@@ -32,12 +36,6 @@
 #include "parse.h"
 #include "userauth.h"
 
-/* For uid_t and gid_t */
-
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #define GABA_DECLARE
 #include "server_userauth.h.x"
 #undef GABA_DECLARE
@@ -54,7 +52,7 @@
 #define USER_LOOKUP(db, n, f) ((db)->lookup((db), (n), (f)))
 
 struct user_db *
-make_unix_user_db(struct reap *reaper,
+make_unix_user_db(struct reaper *reaper,
 		  const char *pw_helper, const char *login_shell,
 		  int allow_root);
 
@@ -66,7 +64,7 @@ make_unix_user_db(struct reap *reaper,
                      "struct ssh_connection *connection"
 		     ; The name is consumed by this function
 		     "struct lsh_string *username"
-		     "UINT32 service"
+		     "uint32_t service"
 		     "struct simple_buffer *args"
 		     "struct command_continuation *c"
 		     "struct exception_handler *e")))
