@@ -1,6 +1,4 @@
 /* server_x11.c
- *
- * $id:$
  *
  */
 
@@ -23,10 +21,27 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <string.h>
+
+#include <fcntl.h>
+#include <unistd.h>
+
+#include <sys/types.h>
+
+#include <sys/socket.h>
+#include <sys/stat.h>
+#include <sys/un.h>
+
 #include "server_x11.h"
 
 #include "channel_commands.h"
 #include "channel_forward.h"
+#include "environ.h"
 #include "format.h"
 #include "reaper.h"
 #include "resource.h"
@@ -34,17 +49,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-#include <string.h>
-
-#include <unistd.h>
-#include <fcntl.h>
-
-#include <sys/types.h>
-
-#include <sys/socket.h>
-#include <sys/stat.h>
-#include <sys/un.h>
 
 #define GABA_DEFINE
 #include "server_x11.h.x"
@@ -79,7 +83,7 @@ struct command open_forwarded_x11;
 static struct ssh_channel *
 new_x11_channel(struct channel_open_command *c,
 		struct ssh_connection *connection,
-		UINT32 local_channel_number,
+		uint32_t local_channel_number,
 		struct lsh_string **request)
 {
   CAST(channel_open_command_x11, self, c);
@@ -401,7 +405,7 @@ make_xauth_exit_callback(struct command_continuation *c,
  * properly authenticated to call it with arbitrary commands. We still
  * check for NUL characters, though. */
 static int
-bad_string(UINT32 length, const UINT8 *data)
+bad_string(uint32_t length, const uint8_t *data)
 {
   return !!memchr(data, '\0', length);
 }
@@ -415,9 +419,9 @@ bad_string(UINT32 length, const UINT8 *data)
 struct server_x11_info *
 server_x11_setup(struct ssh_channel *channel, struct lsh_user *user,
 		 int single,
-		 UINT32 protocol_length, const UINT8 *protocol,
-		 UINT32 cookie_length, const UINT8 *cookie,
-		 UINT32 screen,
+		 uint32_t protocol_length, const uint8_t *protocol,
+		 uint32_t cookie_length, const uint8_t *cookie,
+		 uint32_t screen,
 		 struct command_continuation *c,
 		 struct exception_handler *e)
 {
@@ -456,7 +460,7 @@ server_x11_setup(struct ssh_channel *channel, struct lsh_user *user,
   if (!socket)
     return NULL;
 
-  tmp = getenv("TMPDIR");
+  tmp = getenv(ENV_TMPDIR);
   if (!tmp)
     tmp = "/tmp";
   
