@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: commands.c,v 1.10 2002/05/26 10:58:07 _cvs_pont Exp $
+ * @(#) $Id: commands.c,v 1.14 2003/02/23 18:23:54 nisse Exp $
  *
  * commands.c
  */
@@ -240,7 +240,7 @@ int
 com_longhelp(const char *arg, const char *command UNUSED) 
 {
   const char *s;
-  char tmp[PATH_MAX];
+  char* tmp;
   char* cmdname;
   int i=0;
   
@@ -267,10 +267,10 @@ com_longhelp(const char *arg, const char *command UNUSED)
 
       i=0;
       /* Get the first word from the line (the command) */
-      s = lsftp_s_strtok(s," \n\t\r", tmp, PATH_MAX); 
+      s = lsftp_s_strtok(s," \n\t\r", &tmp ); 
 
   
-      if( !s ) /* No more? */
+      if( !tmp ) /* No more? */
 	return 0;
 
       while( ( cmdname = commands[i++].name) )
@@ -288,6 +288,8 @@ com_longhelp(const char *arg, const char *command UNUSED)
   
       if( !known ) /* If no command matched, say so */
 	printf("Unknown command %s\n", tmp);
+      
+      free( tmp );
     }
 
   return 0; /* Placeholder to stop complaints from compilers*/
@@ -303,19 +305,21 @@ com_mail(const char *arg UNUSED, const char *command)
 int
 com_umask(const char *arg, const char *command UNUSED) 
 {
-  unsigned char tmp[PATH_MAX];
+  char* tmp;
   int given = 0;
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       int newmask = 0;
 
       given++; /* User gave a new mask */
 
-      if( isdigit( tmp[0] ) )
+      /* FIXME: Should probably explicitly state signed/unsigned in all declarations. */
+
+      if( isdigit( (unsigned char) tmp[0] ) )
 	{
 	  newmask = atol( tmp );
 	  umask( newmask );
@@ -323,6 +327,7 @@ com_umask(const char *arg, const char *command UNUSED)
       else
 	printf( "Sorry, only numeric masks allowed\n" );
 
+      free( tmp );
     }
 
   if( !given )
@@ -397,8 +402,8 @@ com_open(const char *arg, const char *command UNUSED)
 {
   char** myargv;
   char** freeargv;
-  const char *s;
-  char tmp[PATH_MAX]; /* Used for arguments */
+  char *s;
+  char* tmp; /* Used for arguments */
 
   int argcount = 0;
   int maxargs;
@@ -420,9 +425,9 @@ com_open(const char *arg, const char *command UNUSED)
       while( argcount < maxargs &&  /* Still room in our array */
 	     arg && 
 	     *arg && 
-	     ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	     ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	     )
-	myargv[argcount++] = strdup( tmp );
+	myargv[argcount++] = tmp;
 
       if( argcount < maxargs )      /* Exited not because the array was full */
 	myargv[argcount] = 0;
@@ -440,7 +445,7 @@ com_open(const char *arg, const char *command UNUSED)
 
       while( (s = *(freeargv++) ) )
 	/* Free doesn't take const pointers */
-	free((void *) s );
+	free( s );
       
       free( myargv );
 
@@ -462,7 +467,7 @@ com_open(const char *arg, const char *command UNUSED)
 int
 com_cd(const char *arg, const char *command UNUSED) 
 {
-  char tmp[PATH_MAX];
+  char* tmp;
 
   int didcd = 0;
 
@@ -473,13 +478,15 @@ com_cd(const char *arg, const char *command UNUSED)
 	 !didcd &&
 	 arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       const char **mglob;
       const char *ptr = 0;
       char* tmp2 = lsftp_qualify_path( tmp );
 
+      free( tmp );
+
       if( !tmp2 )
 	return -1;
 
@@ -512,7 +519,7 @@ com_cd(const char *arg, const char *command UNUSED)
 
 int com_mkdir(const char *arg, const char *command) 
 {
-  char tmp[PATH_MAX];
+  char* tmp;
 
   int id;
   int mode = 0755;
@@ -524,11 +531,13 @@ int com_mkdir(const char *arg, const char *command)
   while( !fail &&
 	 arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       id = lsftp_do_mkdir( tmp, mode, command ); /* FIXME: Default mode? */
       fail = ( -1 == lsftp_await_command( id ) );
+
+      free( tmp );
     }
 
   return 0;
@@ -642,7 +651,7 @@ rm_file_or_folder(const char *path, const char *command, int recursive )
 int
 com_rm(const char *arg, const char *command) 
 {
-  char tmp[PATH_MAX];
+  char* tmp;
 
   int recurse = 0;
   int globbing = 0;
@@ -652,51 +661,55 @@ com_rm(const char *arg, const char *command)
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
-    if( lsftp_get_opts( "rg", tmp )  )
-      switch( lsftp_get_opts( "rg", tmp ) )
-	{
-	case 3: /* Both set? */
-	  recurse = !recurse;
-	  globbing = !globbing;
-	  break;
-	case 2: /* -r */
-	  recurse = !recurse;
-	  break;
-	case 1: /* -g */
-	  globbing = !globbing;
-	  break;
-	default:
-	  printf( "Unknown option(s) %s\n", tmp );
-	}
-    else 
-      {
-	if( !globbing )
-	  rm_file_or_folder( tmp, command, recurse );
-	else      /* Globbing desired */
+    {
+      if( lsftp_get_opts( "rg", tmp )  )
+	switch( lsftp_get_opts( "rg", tmp ) )
 	  {
-	    const char **mglob;
-	    const char **orgglob;
-	    const char *ptr;
-	    
-	    char* tmp2 = lsftp_qualify_path( tmp );
-
-	    if( !tmp2 )
-	      return -1;
-
-
-	    mglob = lsftp_dc_r_startglob( tmp2, 0, 1 );
-	    free( tmp2 );
-	    orgglob = mglob;
-
-	    if( mglob ) /* Glob returned allright? */		
-	      while( (ptr = *mglob++) ) 
-		rm_file_or_folder( ptr, command, recurse );
-	    
-	    lsftp_dc_endglob( orgglob );
+	  case 3: /* Both set? */
+	    recurse = !recurse;
+	    globbing = !globbing;
+	    break;
+	  case 2: /* -r */
+	    recurse = !recurse;
+	    break;
+	  case 1: /* -g */
+	    globbing = !globbing;
+	    break;
+	  default:
+	    printf( "Unknown option(s) %s\n", tmp );
 	  }
-      }
+      else 
+	{
+	  if( !globbing )
+	    rm_file_or_folder( tmp, command, recurse );
+	  else      /* Globbing desired */
+	    {
+	      const char **mglob;
+	      const char **orgglob;
+	      const char *ptr;
+	    
+	      char* tmp2 = lsftp_qualify_path( tmp );
+
+	      if( !tmp2 )
+		return -1;
+
+
+	      mglob = lsftp_dc_r_startglob( tmp2, 0, 1 );
+	      free( tmp2 );
+	      orgglob = mglob;
+
+	      if( mglob ) /* Glob returned allright? */		
+		while( (ptr = *mglob++) ) 
+		  rm_file_or_folder( ptr, command, recurse );
+	    
+	      lsftp_dc_endglob( orgglob );
+	    }
+	}
+
+      free( tmp );
+    }
 
 
   return 0;
@@ -705,7 +718,7 @@ com_rm(const char *arg, const char *command)
 int
 com_ls(const char *arg, const char *command) 
 {
-  char tmp[PATH_MAX];
+  char* tmp;
 
   int didls = 0;
   int longlist = ls_longlist;
@@ -717,7 +730,7 @@ com_ls(const char *arg, const char *command)
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       if( lsftp_get_opts( "la", tmp )  ) 
@@ -742,9 +755,9 @@ com_ls(const char *arg, const char *command)
 	  didls++;
 	  
 	  lsftp_await_command( id );
-
-/*  	  printf( "Returned from command\n" ); */
 	} 
+
+      free( tmp );
     }
   
   if( !didls ) /* No argument given? */
@@ -760,13 +773,13 @@ com_ls(const char *arg, const char *command)
 int
 com_set(const char *arg, const char *command UNUSED) 
 {
-  char tmp[PATH_MAX];
+  char* tmp;
 
   int show = 1;
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       if( !strncmp( "sloppy_complete", tmp, strlen( "sloppy_complete" ) ) )
@@ -848,6 +861,8 @@ com_set(const char *arg, const char *command UNUSED)
 
       if( show ) /* Obviously something was given, but we didn't recognise it */
 	printf( "Unknown command or variable: %s\n", tmp );
+
+      free( tmp );
     }
   
   if( show ) /* No argument given? */
@@ -866,17 +881,19 @@ static int
 get_file_or_folder(const char *arg, const char *command,
 		   int cont, const char *destname )
 {
+  /* FIXME: Split into smaller functions get_file and get_folder used
+   * by this function. */
 
   const char **dirinfo = NULL;
   const char **curdirinfo;
 
-  const char *localwd;
-  const char *remotewd;
+  char *localwd;
+  char *remotewd;
 
-  const char *tmp;
-  char buffer[PATH_MAX+1];
+  const char* realdest;
+  char *tmp;
 
-  struct stat st;
+  struct sftp_attrib attrib;
   int id;
   int mode = 0700;
   int mask;
@@ -914,29 +931,34 @@ get_file_or_folder(const char *arg, const char *command,
   
   /* FIXME: What do we expect filename_part( "foobar/" ) to return? */
   
-  id = lsftp_do_stat( arg, &st );
+  id = lsftp_do_stat( arg, &attrib );
   
   if( id > 0) /* Not a failure? */
     {
       lsftp_await_command( id );
-      mode = st.st_mode;
 
-      if( -1 == mode )
-	mode = 0700;       /* FIXME: Default mode */
+      if (attrib.flags & SSH_FILEXFER_ATTR_PERMISSIONS)
+	mode = attrib.permissions;
+      else
+	/* FIXME: Default mode */
+	mode = 0700;       
     }
 
-  tmp = filename_part( arg ); /* Destination */
-
   if( destname )              /* If given, use it */
-    tmp = destname;
+    realdest = destname;
+  else
+    realdest = filename_part( arg ); /* Destination */
 
-  ret = lsftp_dc_l_isdir( tmp );
+  if( !realdest )              /* Just to be safe */
+    return -1;
+
+  ret = lsftp_dc_l_isdir( realdest );
 
   if( !ret )       /* Exists, but not a dir? */
     {
       printf( 
 	     "Error, %s exists but is not a directory\n", 
-	      tmp
+	      realdest
 	     );
       return -1;
     }
@@ -947,7 +969,7 @@ get_file_or_folder(const char *arg, const char *command,
     umask( mask );
 
 
-    if( mkdir( tmp, (mode & ~mask) | 0700 ) )  /* Make sure we have proper access */
+    if( mkdir( realdest, (mode & ~mask) | 0700 ) )  /* Make sure we have proper access */
       {
 	/* Problem while creating the directory? */
 	
@@ -956,15 +978,15 @@ get_file_or_folder(const char *arg, const char *command,
       }
   }
   
-  ret = lsftp_dc_l_isdir( tmp ); /* Check status */
+  ret = lsftp_dc_l_isdir( realdest ); /* Check status */
   
   if( 1 != ret ) /* If not directory, bail out */
     {
-      printf( "Unable to create directory %s\n", tmp );
+      printf( "Unable to create directory %s\n", realdest );
       return ret;
     }
   
-  localwd = realpath( tmp, buffer ); /* FIXME: Should check boundary in configure somehow */  
+  localwd = canonicalize_file_name( realdest );
 
   if( !destname ) /* No destination given => likely first call,
 		   * make sure arg is an absolute path.
@@ -999,6 +1021,7 @@ get_file_or_folder(const char *arg, const char *command,
   /* Place trailing slashes after both paths */
 
   tmp = lsftp_concat( localwd, "/" );
+  free( localwd );
 
   if( !tmp )
     {
@@ -1006,8 +1029,6 @@ get_file_or_folder(const char *arg, const char *command,
       return -1;
     }
 
-  /* localwd is a static buffer, so don't free it */
-
   localwd = tmp;
 
   tmp = lsftp_concat( remotewd, "/" );
@@ -1018,7 +1039,7 @@ get_file_or_folder(const char *arg, const char *command,
       return -1;
     }
 
-  free( (void *) remotewd );
+  free( remotewd );
   remotewd = tmp;
   
   /* Do globbing */
@@ -1027,14 +1048,18 @@ get_file_or_folder(const char *arg, const char *command,
   
   if( tmp )
     dirinfo = lsftp_dc_r_startglob( tmp, 0, 1 );
-  /* free doesn't accept const poitners */
-  free( (void *) tmp );
+  else
+    return -1; /*Bail out on error */
+
+  free(  tmp );
   
   tmp = lsftp_concat( remotewd, ".*" );
   if( tmp )
     dirinfo = lsftp_dc_r_contglob( tmp, dirinfo, 1 );
-  /* free doesn't accept const pointers */
-  free( (void *) tmp );
+  else
+    return -1; /*Bail out on error */
+  
+  free( tmp );
 
   if( dirinfo )
     {
@@ -1073,8 +1098,8 @@ get_file_or_folder(const char *arg, const char *command,
       lsftp_dc_endglob( dirinfo );
     }
   
-  free( (void *) remotewd ); /* Free memory used by to hold remote wd */
-  free( (void *) localwd ); /* Free memory used by to hold remote wd */
+  free(  remotewd ); /* Free memory used by to hold remote wd */
+  free(  localwd ); /* Free memory used by to hold remote wd */
 
   return 0;
 
@@ -1084,46 +1109,49 @@ int
 com_get(const char *arg, const char *command) 
 {
   int cont = gp_cont;
-  char tmp[PATH_MAX];
+  char* tmp;
 
   if( !lsftp_connected() )   /* Bail out if not connected */
     return com_disconnected();
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
-    if( lsftp_get_opts( "c", tmp )  ) 
-      switch( lsftp_get_opts( "c", tmp ) )
+    {
+      if( lsftp_get_opts( "c", tmp )  ) 
+	switch( lsftp_get_opts( "c", tmp ) )
+	  {
+	  case 1:
+	    cont = !cont;
+	    break;
+	  default:
+	    printf( "Unknown option(s) %s\n", tmp );
+	  }
+      else
 	{
-	case 1:
-	  cont = !cont;
-	  break;
-	default:
-	  printf( "Unknown option(s) %s\n", tmp );
-	}
-    else
-      {
-	const char **glob;
-	const char **orgglob;
-	const char *ptr;
-
-	char* tmp2 = lsftp_qualify_path( tmp );
-
-	if( !tmp2 )
-	  return -1;
-
-	glob = lsftp_dc_r_startglob( tmp2, 0, 1 );
-	free( tmp2 );
-
-	orgglob = glob;
-	
-	while( (ptr = *glob++) ) 
-	  get_file_or_folder( ptr, command, cont, 0 );
+	  const char **glob;
+	  const char **orgglob;
+	  const char *ptr;
 	  
-	lsftp_dc_endglob( orgglob );
-      }
- 
+	  char* tmp2 = lsftp_qualify_path( tmp );
+	  
+	  if( !tmp2 )
+	    return -1;
+	  
+	  glob = lsftp_dc_r_startglob( tmp2, 0, 1 );
+	  free( tmp2 );
+	  
+	  orgglob = glob;
+	  
+	  while( (ptr = *glob++) ) 
+	    get_file_or_folder( ptr, command, cont, 0 );
+	  
+	  lsftp_dc_endglob( orgglob );
+	}
+      
+      free( tmp );
+    }
   return 0;
 }
 
@@ -1131,15 +1159,19 @@ static int
 put_file_or_folder(const char *arg, const char *command,
 		   int cont, const char *destname )
 {
+
+  /* FIXME: Split into smaller functions put_file and put_folder used
+   * by this function. */
+
   const char **dirinfo = NULL;
   const char **curdirinfo;
 
-  const char *localwd;
-  const char *remotewd;
-
-  const char *tmp;
-  char buffer[PATH_MAX];
+  char *localwd;
+  char *remotewd;
 
+  const char* realdest;
+  char *tmp;
+ 
   struct stat st;
   int id;
   int mode = 0700;
@@ -1183,41 +1215,44 @@ put_file_or_folder(const char *arg, const char *command,
     mode = st.st_mode; /* Assign same modes as we have */
   else
     mode = 0700;       /* FIXME: Default mode */
-    
-  tmp = filename_part( arg ); /* Destination */
 
   if( destname )              /* If given, use it */
-    tmp = destname;
+    realdest = destname;
+  else
+    realdest = filename_part( arg ); /* Destination */
 
-  ret = lsftp_dc_r_isdir( tmp );
+  if( !realdest )    /* Just to be safe, bail out on error */
+    return -1;
+
+  ret = lsftp_dc_r_isdir( realdest );
   
   if( !ret )       /* Exists, but not a dir? */
     {
       printf( 
 	     "Error, %s exists but is not a directory\n", 
-	      tmp
+	     realdest
 	     );
       return -1;
     }
   
   if( -1 == ret ) /* stat failed (probably because no such dir) */
   {
-    id = lsftp_do_mkdir( tmp, mode, command ); /* Make directory */
+    id = lsftp_do_mkdir( realdest, mode, command ); /* Make directory */
     
     if( id > 0) /* Not a failure? */
       ret = lsftp_await_command( id );
 
-    ret = lsftp_dc_r_isdir( tmp ); /* Check status */
+    ret = lsftp_dc_r_isdir( realdest ); /* Check status */
 
     if( 1 != ret ) /* If not directory, bail out */
       {
-	printf( "Unable to create directory %s\n", tmp );
+	printf( "Unable to create directory %s\n", realdest );
 	return ret;
       }
 
   }
 
-  localwd = realpath( arg, buffer );
+  localwd = canonicalize_file_name( arg );
 
   if( !destname ) /* No destination given => Make path absolute */
     {
@@ -1253,6 +1288,7 @@ put_file_or_folder(const char *arg, const char *command,
   /* Place trailing slashes after both paths */
 
   tmp = lsftp_concat( localwd, "/" );
+  free( tmp );
 
   if( !tmp )
     {
@@ -1260,8 +1296,6 @@ put_file_or_folder(const char *arg, const char *command,
       return -1;
     }
 
-  /* localwd is a static buffer, so don't free it */
-
   localwd = tmp;
 
   tmp = lsftp_concat( remotewd, "/" );
@@ -1272,7 +1306,7 @@ put_file_or_folder(const char *arg, const char *command,
       return -1;
     }
 
-  free( (void *) remotewd );
+  free( remotewd );
   remotewd = tmp;
   
   /* Do globbing */
@@ -1281,12 +1315,19 @@ put_file_or_folder(const char *arg, const char *command,
   
   if( tmp )
     dirinfo = lsftp_dc_l_startglob( tmp, 1 );
-  free( (void *) tmp );
+  else
+    return -1; /*Bail out on error */
+
+  free( tmp );
   
   tmp = lsftp_concat( localwd, ".*" );
+
   if( tmp )
     dirinfo = lsftp_dc_l_contglob( tmp, dirinfo, 1 );
-  free( (void *) tmp );
+  else
+    return -1; /*Bail out on error */
+
+  free( tmp );
 
   if( dirinfo )
     {
@@ -1326,8 +1367,8 @@ put_file_or_folder(const char *arg, const char *command,
       lsftp_dc_endglob( dirinfo );
     }
   
-  free( (void *) remotewd ); /* Free memory used by to hold remote wd */
-  free( (void *) localwd ); /* Free memory used by to hold remote wd */
+  free( remotewd ); /* Free memory used by to hold remote wd */
+  free( localwd ); /* Free memory used by to hold remote wd */
 
   return 0;
 }    
@@ -1336,7 +1377,7 @@ put_file_or_folder(const char *arg, const char *command,
 
 int com_put(const char *arg, const char *command) 
 {
-  char tmp[PATH_MAX];
+  char* tmp;
   int cont = gp_cont;
 
   if( !lsftp_connected() )   /* Bail out if not connected */
@@ -1344,104 +1385,111 @@ int com_put(const char *arg, const char *command)
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
-    if( lsftp_get_opts( "c", tmp )  ) 
-      switch( lsftp_get_opts( "c", tmp ) )
+    {
+      if( lsftp_get_opts( "c", tmp )  ) 
+	switch( lsftp_get_opts( "c", tmp ) )
+	  {
+	  case 1:
+	    cont = !cont;
+	    break;
+	  default:
+	    printf( "Unknown option(s) %s\n", tmp );
+	  }
+      else
 	{
-	case 1:
-	  cont = !cont;
-	  break;
-	default:
-	  printf( "Unknown option(s) %s\n", tmp );
-	}
-    else
-      {
-	const char **glob;
-	const char **orgglob;
-	const char *ptr;
-	
-	glob = lsftp_dc_l_startglob( tmp, 1 );
-	orgglob = glob;
-	
-	while( (ptr = *glob++) ) 
-	  put_file_or_folder( ptr, command, cont, 0 );
+	  const char **glob;
+	  const char **orgglob;
+	  const char *ptr;
 	  
-	lsftp_dc_endglob( orgglob );
-      }
+	  glob = lsftp_dc_l_startglob( tmp, 1 );
+	  orgglob = glob;
+	  
+	  while( (ptr = *glob++) ) 
+	    put_file_or_folder( ptr, command, cont, 0 );
+	  
+	  lsftp_dc_endglob( orgglob );
+	}
+      
+      free( tmp );
+    }
   
- 
   return 0;
 }
 
 
 int com_chown(const char *arg, const char *command) 
 {
-  unsigned char tmp[PATH_MAX];
+  char* tmp;
   int gotuid = 0;
   int enough_parameters = 0;
   long newuid = 0;
-  struct stat st;
+  struct sftp_attrib attrib;
 
   if( !lsftp_connected() )   /* Bail out if not connected */
     return com_disconnected();
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
-    if( !gotuid ) 
-      {
-	if( isdigit( tmp[0] ) ) /* Numeric GID? FIXME: Better check? */
-	  newuid = atol( tmp );
-	else
-	  {
-	    struct passwd* pwd;
+    {
+      if( !gotuid ) 
+	{
+	  if( isdigit( tmp[0] ) ) /* Numeric GID? FIXME: Better check? */
+	    newuid = atol( tmp );
+	  else
+	    {
+	      struct passwd* pwd;
 
-	    printf( "Warning: performing LOCAL lookup of user %s\n", tmp ); /* Warn user */
+	      printf( "Warning: performing LOCAL lookup of user %s\n", tmp ); /* Warn user */
 	    
-	    pwd = getpwnam( tmp );
+	      pwd = getpwnam( tmp );
 
-	    if( pwd ) /* OK? */
-	      newuid = pwd->pw_uid; /* get uid */
-	    else
-	      {
-		printf( "Lookup failed!\n" );
-		return -1; /* Signal failure */
-	      }
-	  }
+	      if( pwd ) /* OK? */
+		newuid = pwd->pw_uid; /* get uid */
+	      else
+		{
+		  printf( "Lookup failed!\n" );
+		  return -1; /* Signal failure */
+		}
+	    }
 
-	gotuid = 1;
-      }
-    else
-      {
-	int id;
+	  gotuid = 1;
+	}
+      else
+	{
+	  int id;
 	
-	const char **glob;
-	const char **orgglob;
-	const char *ptr;
+	  const char **glob;
+	  const char **orgglob;
+	  const char *ptr;
 
-	enough_parameters++;
+	  enough_parameters++;
 	
-	glob = lsftp_dc_l_startglob( tmp, 1 );
-	orgglob = glob;
+	  glob = lsftp_dc_l_startglob( tmp, 1 );
+	  orgglob = glob;
 	
-	while( (ptr = *glob++) )
-	  {
-	    id = lsftp_do_stat( ptr, &st );
+	  while( (ptr = *glob++) )
+	    {
+	      id = lsftp_do_stat( ptr, &attrib );
+	      if( id > 0) /* Not a failure? */
+		{
+		  lsftp_await_command( id );
 
-       	    if( id > 0) /* Not a failure? */
-	      {
-		lsftp_await_command( id );
-		lsftp_do_chown( ptr, newuid, st.st_gid, command );
-	      }
-	    
-	    free( (void *) ptr );
-	  }
+		  if (! (attrib.flags & SSH_FILEXFER_ATTR_UIDGID))
+		    printf("Remote system doesn't support uids?\n");
+		  
+		  lsftp_do_chown(ptr, newuid, attrib.gid, command );
+		}
+	    }
 	
-	free( orgglob );
-      }
+	  lsftp_dc_endglob( orgglob );
+	}
 	
+      free( tmp );
+    }
 
   if( !enough_parameters )
     {
@@ -1455,56 +1503,62 @@ int com_chown(const char *arg, const char *command)
 
 int com_chgrp(const char *arg, const char *command) 
 {
-  unsigned char tmp[PATH_MAX];
+  char* tmp;
   int gotgid = 0;
   long newgid = 0;
   int enough_parameters = 0;
-  struct stat st;
+  struct sftp_attrib attrib;
 
   if( !lsftp_connected() )   /* Bail out if not connected */
     return com_disconnected();
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
-    if( !gotgid )  /* First time? */
-      {
-	if( isdigit( tmp[0] ) ) /* Numeric GID? FIXME: Better check? */
-	  newgid = atol( tmp );
-	else
-	  {
-	    struct group* gr;
+    {
+      if( !gotgid )  /* First time? */
+	{
+	  if( isdigit( tmp[0] ) ) /* Numeric GID? FIXME: Better check? */
+	    newgid = atol( tmp );
+	  else
+	    {
+	      struct group* gr;
 
-	    printf("Warning: performing LOCAL lookup of group %s\n", tmp ); /* Warn user */
+	      printf("Warning: performing LOCAL lookup of group %s\n", tmp ); /* Warn user */
 	    
-	    gr = getgrnam( tmp );
+	      gr = getgrnam( tmp );
 
-	    if( gr ) /* OK? */
-	      newgid = gr->gr_gid; /* Get gid */
-	    else
-	      {
-		printf("Lookup failed!\n");
-		return -1; /* Signal failure */
-	      }
-	  }
+	      if( gr ) /* OK? */
+		newgid = gr->gr_gid; /* Get gid */
+	      else
+		{
+		  printf("Lookup failed!\n");
+		  return -1; /* Signal failure */
+		}
+	    }
 
-	gotgid = 1;
-      }
-    else
-      {
-	int id;
+	  gotgid = 1;
+	}
+      else
+	{
+	  int id;
 
-	enough_parameters++;
-	id = lsftp_do_stat( tmp, &st );
+	  enough_parameters++;
+	  id = lsftp_do_stat( tmp, &attrib );
 
-	if( id > 0) /* Not a failure? */
-	  {
-	    lsftp_await_command( id );
-	  
-	    lsftp_do_chown( tmp, st.st_uid, newgid, command );
-	  }
-      }
+	  if( id > 0) /* Not a failure? */
+	    {
+	      lsftp_await_command( id );
+
+	      if (! (attrib.flags & SSH_FILEXFER_ATTR_UIDGID))
+		printf("Remote system doesn't support gids?\n");
+	      lsftp_do_chown( tmp, attrib.uid, newgid, command );
+	    }
+	}
+
+      free( tmp );
+    }
 
   if( !enough_parameters )
     {
@@ -1519,11 +1573,11 @@ int
 com_jobs(const char *arg, const char *command UNUSED)
 {
   int listjobs = 0;
-  char tmp[PATH_MAX];
+  char* tmp;
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       int jobid;
@@ -1531,6 +1585,8 @@ com_jobs(const char *arg, const char *command UNUSED)
       jobid = atoi( tmp );
       lsftp_cb_status( jobid ); 
       listjobs++;
+
+      free( tmp );
     }
 
   if( !listjobs )
@@ -1558,24 +1614,26 @@ com_pwd(const char *arg UNUSED, const char *command UNUSED)
 int
 com_mv(const char *arg, const char *command) 
 {
-  char tmp[PATH_MAX];
-  char dst[PATH_MAX+1];
+  char* tmp;
+  char* dst = NULL;
   const char* orgarg = arg;
   int i = 0;
   int j = 0;
-  const char **glob;
-  const char **orgglob;
-  const char *ptr;  
+  const char** glob;
+  const char** orgglob;
+  const char* ptr;  
+  const char* lastptr = NULL;
 
   if( !lsftp_connected() )   /* Bail out if not connected */
     return com_disconnected();
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       char* tmp2 = lsftp_qualify_path( tmp );
+      free( tmp );
 
       if( !tmp2 )
 	return -1;
@@ -1584,15 +1642,25 @@ com_mv(const char *arg, const char *command)
       free( tmp2 );
 
       orgglob = glob;
-      
+      lastptr = NULL;
+
       while( (ptr = *glob++) ) 
 	{
-	  strncpy( dst, ptr, PATH_MAX );
+	  lastptr = ptr;
 	  i++;
 	}
 
+      if( lastptr ) /* Anything matched? */
+	{
+	  
+	  if( dst )
+	    free( dst );
+	  
+	  dst = strdup( lastptr ); /* dst should be a copy of the last argument */	 
+	}
+      
       lsftp_dc_endglob( orgglob );
-      }
+    }
   
   if( i <= 1 ) /* To few arguments */
     {
@@ -1600,15 +1668,20 @@ com_mv(const char *arg, const char *command)
       return -1;
     }
   
+  if( !dst ) /* Found no destination? */
+    return -1;
+  
   arg = orgarg;
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       char* tmp2 = lsftp_qualify_path( tmp );
 
+      free( tmp );
+
       if( !tmp2 )
 	return -1;
 
@@ -1646,20 +1719,21 @@ com_mv(const char *arg, const char *command)
 	      else
 		lsftp_do_mv( ptr, dst, command );
 	    }
-	  free( (void *) ptr );
 	}
 
-      free( orgglob );
+      lsftp_dc_endglob( orgglob );
       }
 
+  free( dst );
+
   return 0;
 }
 
 int
 com_ln(const char *arg, const char *command) 
 {
-  char link[PATH_MAX];
-  char tmp[PATH_MAX];
+  char* link = NULL;
+  char* tmp;
 
   const char *orgarg = arg;
   int i = 0;
@@ -1667,27 +1741,47 @@ com_ln(const char *arg, const char *command)
   const char **glob;
   const char **orgglob;
   const char *ptr;  
+  const char *lastptr;
 
   if( !lsftp_connected() )   /* Bail out if not connected */
     return com_disconnected();
 
-
-
-
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
-      glob = lsftp_dc_r_startglob( tmp, 0, 1 );
+      char* tmp2 = lsftp_qualify_path( tmp );
+
+      free( tmp );
+
+      if( !tmp2 )
+	return -1;
+
+      glob = lsftp_dc_r_startglob( tmp2, 0, 1 );
+      free( tmp2 );
+
       orgglob = glob;
-      
+
+      if( !glob )
+	return -1;
+
+      lastptr = NULL;
+
       while( (ptr = *glob++) ) 
 	{
-	  strncpy( link, ptr, PATH_MAX );
+	  lastptr = ptr;
 	  i++;
 	}
-      
+
+      if( lastptr )
+	{
+	  if( link )
+	    free( link );
+
+	  link = strdup( lastptr);
+	}
+
       lsftp_dc_endglob( orgglob );
     }
   
@@ -1707,7 +1801,7 @@ com_ln(const char *arg, const char *command)
        */
 
       lsftp_do_ln( filename_part( link ), link, command );
-
+      free( link );
       return 0;
    }
 
@@ -1721,7 +1815,7 @@ com_ln(const char *arg, const char *command)
 
   while( arg && 
 	 *arg && 
-	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
+	 ( arg = lsftp_s_strtok( arg," \n\t\r", &tmp ) ) 
 	 )
     {
       glob = lsftp_dc_r_startglob( tmp, 0, 1 );
@@ -1751,11 +1845,13 @@ com_ln(const char *arg, const char *command)
 	      else
 		lsftp_do_ln( link, ptr, command );
 	    }
-	  free( (void *) ptr );
 	}
 
-      free( orgglob );
-      }
+      lsftp_dc_endglob( orgglob );
+
+      free( tmp );
+    }
+
 
   return 0;
 }
@@ -1763,31 +1859,35 @@ com_ln(const char *arg, const char *command)
 int
 com_lcd(const char *arg, const char *command UNUSED) 
 {
-  char tmp[PATH_MAX];
-  const char **gldata = 0;
+  char* tmp;
+  const char **gldata = NULL;
   int ret;
+  
 
-  while( 
-	( arg =
-	  lsftp_s_strtok( arg, " \n\t\r", tmp, PATH_MAX )
-	  )
-	) /* Get argument */    
+  arg = lsftp_s_strtok( arg, " \n\t\r", &tmp );  /* Get argument */    
+   
+  if( tmp )
     gldata = lsftp_dc_l_contglob( tmp, gldata, 1 );
+  
+  free( tmp );
+
+  if( !gldata ) /* glob failed? */
+    return -1;
 
   switch( lsftp_dc_numglob( gldata )  )
     {
     case 0:        /* No path given, go home */
       ret = chdir("~");
       break;
-      
+	
     case 1:
       ret = chdir( gldata[0] );
       break;
       
     default:
-      printf( "lcd: To many arguments (expected one, got %d)\n", 
-	      lsftp_dc_numglob( gldata )
-	      );
+	printf( "lcd: To many arguments (expected one, got %d)\n", 
+		lsftp_dc_numglob( gldata )
+		);
     }
   
   lsftp_dc_endglob( gldata );
@@ -1838,7 +1938,7 @@ gidstring( int gid )
 int
 com_lls(const char *arg, const char *command UNUSED) 
 {
-  char tmp[PATH_MAX];
+  char* tmp;
   const char **gldata = 0;
   struct stat st;
   int i = 0;
@@ -1851,7 +1951,7 @@ com_lls(const char *arg, const char *command UNUSED)
 
   while( 
 	( arg =
-	  lsftp_s_strtok( arg, " \n\t\r", tmp, PATH_MAX )
+	  lsftp_s_strtok( arg, " \n\t\r", &tmp )
 	  )
 	) /* Get argument */    
     {
@@ -1876,6 +1976,8 @@ com_lls(const char *arg, const char *command UNUSED)
 	  gldata = lsftp_dc_l_contglob( tmp, gldata, 1 );
 	  args++;
 	}
+
+      free( tmp );
     }
 
   if( !args )
@@ -1988,11 +2090,32 @@ int
 com_lpwd(const char *arg UNUSED, const char *command UNUSED) 
 {
   char* pwd;
-  pwd = getcwd( NULL, PATH_MAX ); /* Get where we are now */
+  char* ret;
+  int size;
 
-  if( !pwd )            /* Failed? */
+#ifdef PATH_MAX
+  size = PATH_MAX;
+#else
+  size = 8192;
+#endif
+
+  /* FIXME: Detect and use if getcwd can allocate the buffer by itself */
+
+  pwd = malloc( size+1 );
+
+  if( !pwd )
     return -1;
 
+  ret = getcwd( pwd, size ); /* Get where we are now */
+  pwd[ size ] = 0; /* Make certain it is NULL-terminated*/
+
+  if( !ret )            /* Failed? */
+    {
+      perror( "getcwd failed" );
+      free( pwd );
+      return -1;
+    }
+
   printf( "%s\n",pwd ); /*  Print it */
   free( pwd );
   return 0;      
@@ -2012,7 +2135,7 @@ int
 handle_command(const unsigned char *s)
 {
   const char *entered_cmd = s;
-  char tmp[WORDLENMAX];
+  char* tmp;
   char *cmdname;
   int i=0;
     
@@ -2032,7 +2155,7 @@ handle_command(const unsigned char *s)
 
   /* Get the first word from the line (the command) */
 
-  s = lsftp_s_strtok(s," \n\t\r", tmp, WORDLENMAX); 
+  s = lsftp_s_strtok(s," \n\t\r", &tmp );
   
   if(!s) /* s equals null ? */
     return NOCOMMAND;
@@ -2047,14 +2170,20 @@ handle_command(const unsigned char *s)
       if( neededlen <= tmplen  &&  /* Sufficiently long string? */
 	  !strncmp( cmdname, tmp, tmplen )  /* Matching command? */                
 	  )          
-	return commands[i-1].func( 
-				lsftp_s_skip(s," \n\t\r"), 
-				entered_cmd 
-				);
-	
+	{
+	  free( tmp );
+
+	  return commands[i-1].func( 
+				    lsftp_s_skip(s," \n\t\r"), 
+				    entered_cmd 
+				    );
+	}
     }
 
+  
   printf("Unknown command %s\n", tmp);
+  
+  free( tmp );
   return -UNKNOWNCOMMAND;
 
 }
