@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: rl.c,v 1.5 2002/05/26 14:33:19 _cvs_pont Exp $
+ * @(#) $Id: rl.c,v 1.8 2003/02/24 20:05:37 nisse Exp $
  *
  * rl.c
  */
@@ -192,7 +192,7 @@ char** lsftp_rl_completion(char* text, int start, int end)
 
   /* If this word is at the start of the line, then it is a command to
    * complete. 
-n   */
+   */
 
   rl_completion_append_character = ' '; /* Trailing space after word */
 
@@ -208,48 +208,54 @@ n   */
   else
     {
       /* We're not completing the command */
-      char tmp[256];
+      char* tmp;
       int i=0;
       char* cmdname;
 
       /* Get the first word from the line (the command) */
 
-      lsftp_s_strtok( rl_line_buffer+s," \n\t\r", tmp, 256 );
+      lsftp_s_strtok( rl_line_buffer+s," \n\t\r", &tmp );
 
       while( (cmdname = commands[i].name) )
-	if ( ! strncmp( cmdname, tmp, commands[i++].uniquelen ) )       /* We've got the command? */
-	  switch ( commands[i-1].arg_type )
-	    {
+	{
+	  if ( ! strncmp( cmdname, tmp, commands[i++].uniquelen ) )       /* We've got the command? */
+	    switch ( commands[i-1].arg_type )
+	      {
+		
+	      case COMMAND: /* Command takes a command as argument? */
+		matches = RL_COMPLETION_MATCHES( 
+						text, 
+						lsftp_rl_command_generator
+						);
+		break;
+		
+	      case REMOTEFILE:  /* Remote file or dir as argument? */
+		matches = RL_COMPLETION_MATCHES( 
+						text, 
+						lsftp_rl_remotefile_generator
+						);
+		break;
+		
+	      case LOCALFILE:
+		matches = RL_COMPLETION_MATCHES(
+						text, 
+						RL_FILENAME_COMPLETION_FUNCTION
+						);
+		break;
+		
+	      default:
+	      case NOARG:  /* Doesn't take any argument? */
+		matches = RL_COMPLETION_MATCHES(
+						text, 
+						lsftp_rl_no_generator 
+						); 
+		break;
+	      }
+	  
+	 
+	}
 
-	    case COMMAND: /* Command takes a command as argument? */
-	      matches = RL_COMPLETION_MATCHES( 
-					      text, 
-					      lsftp_rl_command_generator
-					      );
-	      break;
-
-	    case REMOTEFILE:  /* Remote file or dir as argument? */
-	      matches = RL_COMPLETION_MATCHES( 
-					      text, 
-					      lsftp_rl_remotefile_generator
-					      );
-	      break;
-
-	    case LOCALFILE:
-	      matches = RL_COMPLETION_MATCHES(
-					      text, 
-					      RL_FILENAME_COMPLETION_FUNCTION
-					      );
-	      break;
-
-	    default:
-	    case NOARG:  /* Doesn't take any argument? */
-	      matches = RL_COMPLETION_MATCHES(
-					      text, 
-					      lsftp_rl_no_generator 
-					      ); 
-	      break;
-	    }
+      free( tmp );
 
       if( !matches ) /* No matching command? */
 	  {
@@ -308,7 +314,7 @@ char* lsftp_rl_remotefile_generator( char* text, int state )
 {
   static int list_index;
   char* name;
-  static char** glob;
+  static const char** glob;
   static int unqualify = 0;
 
   /* If this is a new word to complete, initialize now. */
@@ -384,7 +390,7 @@ char* lsftp_rl_remotefile_generator( char* text, int state )
       char* tmp;
       char* tmp2;
       char* nfree = strdup( glob[list_index++] );
-      char* name = nfree; 
+      const char* name = nfree; 
 
       if( unqualify )
 	name = lsftp_unqualify_path( nfree );
