@@ -2,7 +2,7 @@
  *
  * Create an SRP verifier
  *
- * $Id: srp-gen.c,v 1.13 2001/12/12 18:37:54 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -26,11 +26,11 @@
 #include "srp.h"
 
 #include "crypto.h"
+#include "environ.h"
 #include "format.h"
 #include "interact.h"
 #include "io.h"
 #include "randomness.h"
-#include "sexp.h"
 #include "srp.h"
 #include "version.h"
 #include "werror.h"
@@ -67,13 +67,11 @@ const char *argp_program_bug_address = BUG_ADDRESS;
        
        (e object exception_handler)
        (G object abstract_group)
-       (H object hash_algorithm)
+       (H const object hash_algorithm)
        
        (file string)
        (dest object abstract_write)
 
-       (style . sexp_argp_state)
-
        (name . "const char *")
        (passwd string)
        (r object randomness)))
@@ -90,17 +88,15 @@ make_srp_gen_options(struct exception_handler *e)
   self->e = e;
 
   self->G = make_ssh_ring_srp_1();
-  self->H = &sha1_algorithm;
+  self->H = &crypto_sha1_algorithm;
   self->file = NULL;
   self->dest = NULL;
 
-  self->style = -1;
-
-  self->name = getenv("LOGNAME");
+  self->name = getenv(ENV_LOGNAME);
   self->passwd = NULL;
 
   /* We use this only for generating the salt. */
-  self->r = make_user_random(getenv("HOME"));
+  self->r = make_user_random(getenv(ENV_HOME));
 
   return self;
 }
@@ -118,7 +114,6 @@ main_options[] =
 static const struct argp_child
 main_argp_children[] =
 {
-  { &sexp_output_argp, 0, NULL, 0 },
   { &werror_argp, 0, "", 0 },
   { NULL, 0, NULL, 0}
 };
@@ -134,8 +129,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       return ARGP_ERR_UNKNOWN;
 
     case ARGP_KEY_INIT:
-      state->child_inputs[0] = &self->style;
-      state->child_inputs[1] = NULL;
+      state->child_inputs[0] = NULL;
       break;
 
     case ARGP_KEY_END:
@@ -185,8 +179,6 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 
 	  lsh_string_free(again);
 	}
-      if (self->style < 0)
-	self->style = self->file ? SEXP_CANONICAL : SEXP_TRANSPORT;
       
       break;
 
@@ -210,12 +202,12 @@ main_argp =
   NULL, NULL
 };
 
-static struct sexp *
+static struct lsh_string *
 srp_gen(struct srp_gen_options *options)
 {
   struct lsh_string *salt;
   struct lsh_string *name;
-  struct sexp *e;
+  struct lsh_string *res;
 
   /* NOTE: Allows random to be of bad quality */
   salt = lsh_string_alloc(SALT_SIZE);
@@ -223,11 +215,12 @@ srp_gen(struct srp_gen_options *options)
   RANDOM(options->r, salt->length, salt->data);
   name = ssh_format("%lz", options->name);
 
-  e = srp_make_verifier(options->G, options->H,
-			salt, name, options->passwd);
+  /* FIXME: Leaks some strings. */
+  res = srp_make_verifier(options->G, options->H,
+			  salt, name, options->passwd);
   lsh_string_free(name);
 
-  return e;
+  return res;
 }
 
 static void
@@ -251,9 +244,9 @@ int main(int argc, char **argv)
   
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
 
-  A_WRITE(options->dest,
-	  sexp_format(srp_gen(options), options->style, 0));
-
+  /* FIXME: Use write_raw instead. */
+  A_WRITE(options->dest, srp_gen(options));
+  
   io_run();
   io_final();
   
