@@ -2,7 +2,7 @@
  *
  * Handle process suspend for the client programs.
  *
- * $Id: suspend.h,v 1.1 2001/06/19 21:08:49 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,7 +34,7 @@ suspend_install_handler(void);
 void
 suspend_handle_tty(int fd);
 
-struct lsh_callback
+extern struct lsh_callback
 suspend_callback;
 
 #endif /* LSH_SUSPEND_H_INCLUDED */
