@@ -9,7 +9,7 @@ set -e
 : ${LSH_YARROW_SEED_FILE:="$TEST_HOME/.lsh/yarrow-seed-file"}
 
 # For lsh-authorize
-: ${SEXP_CONV:="`pwd`/../sexp-conv"}
+: ${SEXP_CONV:="`pwd`/../nettle/tools/sexp-conv"}
 
 export LSH_YARROW_SEED_FILE SEXP_CONV
 
@@ -20,6 +20,14 @@ export LSH_YARROW_SEED_FILE SEXP_CONV
 : ${PIDFILE:="`pwd`/lshd.$$.pid"}
 : ${INTERFACE:=127.0.0.1}
 
+# Ignore any options the tester might have put in the environment.
+
+# With bash, unset returns a non-zero exit status for non-existing
+# variables. We have to ignore that error.
+
+unset LSHGFLAGS || :
+unset LSHFLAGS || :
+
 PORT=11147
 ATEXIT='set +e'
 
@@ -43,10 +51,25 @@ test_skip () {
     exit
 }
 
+werror () {
+    echo 1>&2 "$1"
+}
+
+die () {
+    werror "$1"
+    test_fail
+}
+
 check_x11_support () {
     ../lsh --help | grep 'x11-forward' >/dev/null || test_skip
 }
 
+need_tcputils () {
+    if type tcpconnect >/dev/null 2>&1 ; then : ; else
+	test_skip
+    fi
+}
+
 trap 'eval "$ATEXIT ; exit \$test_result"' 0
 
 at_exit () {
