@@ -2,7 +2,6 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.55 2002/08/27 17:17:03 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -24,33 +23,27 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "server_userauth.h"
-
-#include "format.h"
-#include "io.h"
-#include "read_file.h"
-#include "reaper.h"
-#include "server_pty.h"
-#include "werror.h"
-#include "xalloc.h"
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <assert.h>
 #include <errno.h>
 #include <string.h>
 #include <time.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#include <sys/wait.h>
-
 #include <signal.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/wait.h>
+
 #if HAVE_CRYPT_H
 # include <crypt.h>
 #endif
@@ -77,6 +70,18 @@
 # include <libutil.h>
 #endif
 
+#include "server_userauth.h"
+
+#include "environ.h"
+#include "format.h"
+#include "io.h"
+#include "read_file.h"
+#include "reaper.h"
+#include "server_pty.h"
+#include "werror.h"
+#include "xalloc.h"
+
+
 /* Forward declaration */
 struct unix_user_db;
 
@@ -420,7 +425,7 @@ make_exc_read_user_file_handler(struct abstract_write *c,
 static const struct exception *
 do_read_file(struct lsh_user *u, 
 	     const char *name, int secret,
-	     UINT32 limit,
+	     uint32_t limit,
 	     struct abstract_write *c)
 {
   CAST(unix_user, user, u);
@@ -585,7 +590,7 @@ exec_shell(struct unix_user *user, struct spawn_info *info)
   const char **shell_argv;
   const char *argv0;
 
-  char *tz = getenv("TZ");
+  char *tz = getenv(ENV_TZ);
   unsigned i, j;
 
   trace("unix_user: exec_shell\n");
@@ -605,18 +610,18 @@ exec_shell(struct unix_user *user, struct spawn_info *info)
   envp = alloca(sizeof(char *) * (info->env_length + MAX_ENV + 1));
 
   i = 0;
-  envp[i++] = format_env_pair("SHELL", user->shell);
+  envp[i++] = format_env_pair(ENV_SHELL, user->shell);
 
   if (user->home)
-    envp[i++] = format_env_pair("HOME", user->home);
+    envp[i++] = format_env_pair(ENV_HOME, user->home);
 
   /* FIXME: The value of $PATH should not be hard-coded */
-  envp[i++] = "PATH=/bin:/usr/bin";
-  envp[i++] = format_env_pair("USER", user->super.name);
-  envp[i++] = format_env_pair("LOGNAME", user->super.name);
+  envp[i++] = ENV_PATH "=/bin:/usr/bin";
+  envp[i++] = format_env_pair(ENV_USER, user->super.name);
+  envp[i++] = format_env_pair(ENV_LOGNAME, user->super.name);
 
   if (tz)
-    envp[i++] = format_env_pair_c("TZ", tz);
+    envp[i++] = format_env_pair_c(ENV_TZ, tz);
 
   assert(i <= MAX_ENV);
 #undef MAX_ENV
@@ -705,7 +710,7 @@ exec_shell(struct unix_user *user, struct spawn_info *info)
   if (user->super.uid == getuid())
     execve(lsh_get_cstring(user->shell), (char **) shell_argv, (char **) envp);
   else
-    execve(PREFIX "/sbin/lsh-execuv", (char **) argv, (char **) envp);
+    execve(PATH_EXECUV, (char **) argv, (char **) envp);
 
   werror("unix_user: exec failed %e\n", errno);
   _exit(EXIT_FAILURE);
@@ -718,6 +723,30 @@ safe_close(int fd)
     werror("close failed %e\n", errno);
 }
 
+/* FIXME: We need some way to report errors when the child process
+ * tries to start the child.
+ *
+ * One simple approach is to reuse the sync pipe for this. Have
+ * do_spawn read a limited amount (say 200 bytes or so) from the pipe.
+ * If it gets exactly zero, then everything is ok. If it get's some
+ * data, that should be returned to the caller in some way.
+ *
+ * In the child process, exec_shell can report errors on that fd. If
+ * it exec's the shell directly, it should first set the close-on-exec
+ * flag. If it uses the lsh-execuv mechanism, it can keep the
+ * close-on-exec-flag cleared, and pass a new command line option
+ * --error-fd to lsh-execuv. lsh-execuv can write error messages
+ * there, and then it has the responsibility of setting the close-on-exec flag.
+ *
+ * A potential problem with this approach is that the lshd server will
+ * hang until lsh-execuv is finished. One can either step back and let
+ * lsh-execuv report errors on the ordinary stderr (simplest), or use
+ * non-blocking mode for reading the syncronization pipe (probably too
+ * complex). Some non-blocking behaviour might be needed anyway,
+ * though, because of the xauth bug that results in channel data being
+ * sent before the reply to a shell request.
+ */
+
 static struct lsh_process *
 do_spawn(struct lsh_user *u,
 	 struct spawn_info *info,
@@ -1018,7 +1047,7 @@ do_lookup_user(struct user_db *s,
       /* NOTE: If we are running as the uid of the user, it seems like
        * a good idea to let the HOME environment variable override the
        * passwd-database. */
-      home = getenv("HOME");
+      home = getenv(ENV_HOME);
       if (!home)
 	home = passwd->pw_dir;
     }
