@@ -1,6 +1,5 @@
 /* userauth.h
  *
- * $Id: userauth.h,v 1.36 2002/08/25 08:14:43 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -25,12 +24,12 @@
 #ifndef LSH_USERAUTH_H_INCLUDED
 #define LSH_USERAUTH_H_INCLUDED
 
-#include "exception.h"
-#include "resource.h"
-
 /* For uid_t */
 #include <unistd.h>
 
+#include "exception.h"
+#include "resource.h"
+
 /* Forward declaration */
 struct env_value;
 struct spawn_info;
@@ -121,7 +120,7 @@ struct spawn_info
        ; it must also not be readable by other users.
        (read_file method "const struct exception *"
                   "const char *name" "int secret"
-		  "UINT32 limit"
+		  "uint32_t limit"
 		  ; Gets the contents of the file
 		  "struct abstract_write *c")
 		  
