@@ -1,8 +1,6 @@
 /* write_buffer.c
  *
- *
- *
- * $Id: write_buffer.c,v 1.22 2001/04/04 21:08:10 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -23,14 +21,20 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "write_buffer.h"
-
-#include "xalloc.h"
-#include "werror.h"
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <assert.h>
 #include <string.h>
 
+#include "write_buffer.h"
+
+#include "io.h"
+#include "xalloc.h"
+#include "werror.h"
+
+
 #define GABA_DEFINE
 #include "write_buffer.h.x"
 #undef GABA_DEFINE
@@ -61,14 +65,12 @@ do_write(struct abstract_write *w,
 
   string_queue_add_tail(&closure->q, packet);
   
-#if 0
-  if (closure->try_write)
-    {
-      /* Attempt writing to the corresponding fd. */
-    }
-#endif
-
+  if (!closure->length)
+    /* We're making the buffer non-empty */
+    lsh_oop_register_write_fd(closure->fd);
+  
   closure->empty = 0;
+  
   closure->length += packet->length;
 
   debug("write_buffer: do_write closure->length = %i\n",
@@ -83,7 +85,7 @@ do_write(struct abstract_write *w,
  * Returns 1 if the buffer is non-empty. */
 int write_buffer_pre_write(struct write_buffer *buffer)
 {
-  UINT32 length = buffer->end - buffer->start;
+  uint32_t length = buffer->end - buffer->start;
 
   if (buffer->empty)
     return 0;
@@ -101,8 +103,8 @@ int write_buffer_pre_write(struct write_buffer *buffer)
       /* Copy more data into buffer */
       if (buffer->partial)
 	{
-	  UINT32 partial_left = buffer->partial->length - buffer->pos;
-	  UINT32 buffer_left = 2*buffer->block_size - buffer->end;
+	  uint32_t partial_left = buffer->partial->length - buffer->pos;
+	  uint32_t buffer_left = 2*buffer->block_size - buffer->end;
 	  if (partial_left <= buffer_left)
 	    {
 	      /* The rest of the partial packet fits in the buffer */
@@ -145,7 +147,7 @@ int write_buffer_pre_write(struct write_buffer *buffer)
   return !buffer->empty;
 }
 
-void write_buffer_consume(struct write_buffer *buffer, UINT32 size)
+void write_buffer_consume(struct write_buffer *buffer, uint32_t size)
 {
   buffer->start += size;
   assert(buffer->start <= buffer->end);
@@ -155,17 +157,14 @@ void write_buffer_consume(struct write_buffer *buffer, UINT32 size)
     FLOW_CONTROL_REPORT(buffer->report, size);
 }
 
-void write_buffer_close(struct write_buffer *buffer)
-{
-  buffer->closed = 1;
-}
-
 struct write_buffer *
-make_write_buffer(UINT32 size)
+make_write_buffer(struct lsh_fd *fd, uint32_t size)
 {
   NEW(write_buffer, res);
-  
+
   res->super.write = do_write;
+
+  res->fd = fd;
   
   res->block_size = size;
 
